'''OpenGL extension ARB.shader_objects

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shader_objects to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds API calls that are necessary to manage shader
	objects and program objects as defined in the OpenGL 2.0 white papers by
	3Dlabs.
	
	The generation of an executable that runs on one of OpenGL's
	programmable units is modeled to that of developing a typical C/C++
	application. There are one or more source files, each of which are
	stored by OpenGL in a shader object. Each shader object (source file)
	needs to be compiled and attached to a program object. Once all shader
	objects are compiled successfully, the program object needs to be linked
	to produce an executable. This executable is part of the program object,
	and can now be loaded onto the programmable units to make it part of the
	current OpenGL state. Both the compile and link stages generate a text
	string that can be queried to get more information. This information
	could be, but is not limited to, compile errors, link errors,
	optimization hints, etc. Values for uniform variables, declared in a
	shader, can be set by the application and used to control a shader's
	behavior.
	
	This extension defines functions for creating shader objects and program
	objects, for compiling shader objects, for linking program objects, for
	attaching shader objects to program objects, and for using a program
	object as part of current state. Functions to load uniform values are
	also defined. Some house keeping functions, like deleting an object and
	querying object state, are also provided.
	
	Although this extension defines the API for creating shader objects, it
	does not define any specific types of shader objects. It is assumed that
	this extension will be implemented along with at least one such
	additional extension for creating a specific type of OpenGL 2.0 shader
	(e.g., the ARB_fragment_shader extension or the ARB_vertex_shader
	extension).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/shader_objects.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.shader_objects import *
from OpenGL.raw.GL.ARB.shader_objects import _EXTENSION_NAME

def glInitShaderObjectsARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glShaderSourceARB.length size not checked against count
# INPUT glShaderSourceARB.string size not checked against count
glShaderSourceARB=wrapper.wrapper(glShaderSourceARB).setInputArraySize(
    'length', None
).setInputArraySize(
    'string', None
)
# INPUT glUniform1fvARB.value size not checked against count
glUniform1fvARB=wrapper.wrapper(glUniform1fvARB).setInputArraySize(
    'value', None
)
# INPUT glUniform2fvARB.value size not checked against count*2
glUniform2fvARB=wrapper.wrapper(glUniform2fvARB).setInputArraySize(
    'value', None
)
# INPUT glUniform3fvARB.value size not checked against count*3
glUniform3fvARB=wrapper.wrapper(glUniform3fvARB).setInputArraySize(
    'value', None
)
# INPUT glUniform4fvARB.value size not checked against count*4
glUniform4fvARB=wrapper.wrapper(glUniform4fvARB).setInputArraySize(
    'value', None
)
# INPUT glUniform1ivARB.value size not checked against count
glUniform1ivARB=wrapper.wrapper(glUniform1ivARB).setInputArraySize(
    'value', None
)
# INPUT glUniform2ivARB.value size not checked against count*2
glUniform2ivARB=wrapper.wrapper(glUniform2ivARB).setInputArraySize(
    'value', None
)
# INPUT glUniform3ivARB.value size not checked against count*3
glUniform3ivARB=wrapper.wrapper(glUniform3ivARB).setInputArraySize(
    'value', None
)
# INPUT glUniform4ivARB.value size not checked against count*4
glUniform4ivARB=wrapper.wrapper(glUniform4ivARB).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix2fvARB.value size not checked against count*4
glUniformMatrix2fvARB=wrapper.wrapper(glUniformMatrix2fvARB).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix3fvARB.value size not checked against count*9
glUniformMatrix3fvARB=wrapper.wrapper(glUniformMatrix3fvARB).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix4fvARB.value size not checked against count*16
glUniformMatrix4fvARB=wrapper.wrapper(glUniformMatrix4fvARB).setInputArraySize(
    'value', None
)
glGetObjectParameterfvARB=wrapper.wrapper(glGetObjectParameterfvARB).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetObjectParameterivARB=wrapper.wrapper(glGetObjectParameterivARB).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetInfoLogARB=wrapper.wrapper(glGetInfoLogARB).setOutput(
    'infoLog',size=lambda x:(x,),pnameArg='maxLength',orPassIn=True
).setOutput(
    'length',size=(1,),orPassIn=True
)
glGetAttachedObjectsARB=wrapper.wrapper(glGetAttachedObjectsARB).setOutput(
    'count',size=(1,),orPassIn=True
).setOutput(
    'obj',size=lambda x:(x,),pnameArg='maxCount',orPassIn=True
)
glGetActiveUniformARB=wrapper.wrapper(glGetActiveUniformARB).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'name',size=lambda x:(x,),pnameArg='maxLength',orPassIn=True
).setOutput(
    'size',size=(1,),orPassIn=True
).setOutput(
    'type',size=(1,),orPassIn=True
)
# OUTPUT glGetUniformfvARB.params COMPSIZE(programObj, location) 
# OUTPUT glGetUniformivARB.params COMPSIZE(programObj, location) 
glGetShaderSourceARB=wrapper.wrapper(glGetShaderSourceARB).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'source',size=lambda x:(x,),pnameArg='maxLength',orPassIn=True
)
### END AUTOGENERATED SECTION
import OpenGL
from OpenGL._bytes import bytes, _NULL_8_BYTE, as_8_bit
from OpenGL.raw.GL import _errors
from OpenGL.lazywrapper import lazy as _lazy
from OpenGL import converters, error
GL_INFO_LOG_LENGTH_ARB = constant.Constant( 'GL_INFO_LOG_LENGTH_ARB', 0x8B84 )

glShaderSourceARB = platform.createExtensionFunction(
    'glShaderSourceARB', dll=platform.PLATFORM.GL,
    resultType=None,
    argTypes=(_types.GLhandleARB, _types.GLsizei, ctypes.POINTER(ctypes.c_char_p), arrays.GLintArray,),
    doc = 'glShaderSourceARB( GLhandleARB(shaderObj), [bytes(string),...] ) -> None',
    argNames = ('shaderObj', 'count', 'string', 'length',),
    extension = _EXTENSION_NAME,
)
conv = converters.StringLengths( name='string' )
glShaderSourceARB = wrapper.wrapper(
    glShaderSourceARB
).setPyConverter(
    'count' # number of strings
).setPyConverter(
    'length' # lengths of strings
).setPyConverter(
    'string', conv.stringArray
).setCResolver(
    'string', conv.stringArrayForC,
).setCConverter(
    'length', conv,
).setCConverter(
    'count', conv.totalCount,
)
try:
    del conv
except NameError as err:
    pass

def _afterCheck( key ):
    """Generate an error-checking function for compilation operations"""
    def GLSLCheckError(
        result,
        baseOperation=None,
        cArguments=None,
        *args
    ):
        result = _errors._error_checker.glCheckError( result, baseOperation, cArguments, *args )
        status = glGetObjectParameterivARB(
            cArguments[0], key
        )
        if not status:
            raise error.GLError(
                result = result,
                baseOperation = baseOperation,
                cArguments = cArguments,
                description= glGetInfoLogARB( cArguments[0] )
            )
        return result
    return GLSLCheckError

if OpenGL.ERROR_CHECKING:
    glCompileShaderARB.errcheck = _afterCheck( GL_OBJECT_COMPILE_STATUS_ARB )
if OpenGL.ERROR_CHECKING:
    glLinkProgramARB.errcheck = _afterCheck( GL_OBJECT_LINK_STATUS_ARB )
## Not sure why, but these give invalid operation :(
##if glValidateProgramARB and OpenGL.ERROR_CHECKING:
##	glValidateProgramARB.errcheck = _afterCheck( GL_OBJECT_VALIDATE_STATUS_ARB )

@_lazy( glGetInfoLogARB )
def glGetInfoLogARB( baseOperation, obj ):
    """Retrieve the program/shader's error messages as a Python string

    returns string which is '' if no message
    """
    length = int(glGetObjectParameterivARB(obj, GL_INFO_LOG_LENGTH_ARB))
    if length > 0:
        log = ctypes.create_string_buffer(length)
        baseOperation(obj, length, None, log)
        return log.value.strip(_NULL_8_BYTE) # null-termination
    return ''

@_lazy( glGetAttachedObjectsARB )
def glGetAttachedObjectsARB( baseOperation, obj ):
    """Retrieve the attached objects as an array of GLhandleARB instances"""
    length= glGetObjectParameterivARB( obj, GL_OBJECT_ATTACHED_OBJECTS_ARB )
    if length > 0:
        storage = arrays.GLuintArray.zeros( (length,))
        baseOperation( obj, length, None, storage )
        return storage
    return arrays.GLuintArray.zeros( (0,))

@_lazy( glGetShaderSourceARB )
def glGetShaderSourceARB( baseOperation, obj ):
    """Retrieve the program/shader's source code as a Python string

    returns string which is '' if no source code
    """
    length = int(glGetObjectParameterivARB(obj, GL_OBJECT_SHADER_SOURCE_LENGTH_ARB))
    if length > 0:
        source = ctypes.create_string_buffer(length)
        baseOperation(obj, length, None, source)
        return source.value.strip(_NULL_8_BYTE) # null-termination
    return ''

@_lazy( glGetActiveUniformARB )
def glGetActiveUniformARB(baseOperation,program, index,bufSize=None):
    """Retrieve the name, size and type of the uniform of the index in the program"""
    max_index = int(glGetObjectParameterivARB( program, GL_OBJECT_ACTIVE_UNIFORMS_ARB ))
    if bufSize is None:
        bufSize = int(glGetObjectParameterivARB( program, GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB))
    if index < max_index and index >= 0:
        length,name,size,type = baseOperation( program, index, bufSize )
        if hasattr(name,'tostring'):
            name = name.tostring().rstrip(b'\000')
        elif hasattr(name,'value'):
            name = name.value
        return name,size,type
    raise IndexError( 'Index %s out of range 0 to %i' % (index, max_index - 1, ) )

@_lazy( glGetUniformLocationARB )
def glGetUniformLocationARB( baseOperation, program, name ):
    """Check that name is a string with a null byte at the end of it"""
    if not name:
        raise ValueError( """Non-null name required""" )
    name = as_8_bit( name )
    if name[-1] != _NULL_8_BYTE:
        name = name + _NULL_8_BYTE
    return baseOperation( program, name )



"""Convenience module providing common shader entry points

The point of this module is to allow client code to use
OpenGL Core names to reference shader-related operations
even if the local hardware only supports ARB extension-based
shader rendering.

There are also two utility methods compileProgram and compileShader
which make it easy to create demos which are shader-using.
"""
import logging
log = logging.getLogger( __name__ )
from OpenGL import GL
from OpenGL.GL.ARB import (
    shader_objects, fragment_shader, vertex_shader, vertex_program,
    geometry_shader4, separate_shader_objects, get_program_binary,
)
from OpenGL.extensions import alternate
from OpenGL._bytes import bytes,unicode,as_8_bit

__all__ = [
    'glAttachShader',
    'glDeleteShader',
    'glGetProgramInfoLog',
    'glGetShaderInfoLog',
    'glGetProgramiv',
    'glGetShaderiv',
    'compileProgram',
    'compileShader',
    'GL_VALIDATE_STATUS',
    'GL_LINK_STATUS',
    'ShaderCompilationError', 
    'ShaderValidationError', 
    'ShaderLinkError',
    # automatically added stuff here...
]

def _alt( base, name ):
    if hasattr( GL, base ):
        root = getattr( GL, base )
        if hasattr(root,'__call__'):
            globals()[base] = alternate(
                getattr(GL,base),
                getattr(module,name)
            )
            __all__.append( base )
        else:
            globals()[base] = root
            __all__.append( base )
        return True
    return False
_excludes = ['glGetProgramiv']
for module in (
    shader_objects,fragment_shader,vertex_shader,vertex_program,
   geometry_shader4,
):
    for name in dir(module):
        found = None
        for suffix in ('ObjectARB','_ARB','ARB'):
            if name.endswith( suffix ):
                found = False
                base = name[:-(len(suffix))]
                if base not in _excludes:
                    if _alt( base, name ):
                        found = True
                        break
        if found is False:
            log.debug( '''Found no alternate for: %s.%s''',
                module.__name__,name,
            )

glAttachShader = alternate( GL.glAttachShader,shader_objects.glAttachObjectARB )
glDetachShader = alternate( GL.glDetachShader,shader_objects.glDetachObjectARB )
glDeleteShader = alternate( GL.glDeleteShader,shader_objects.glDeleteObjectARB )
glGetAttachedShaders = alternate( GL.glGetAttachedShaders, shader_objects.glGetAttachedObjectsARB )

glGetProgramInfoLog = alternate( GL.glGetProgramInfoLog, shader_objects.glGetInfoLogARB )
glGetShaderInfoLog = alternate( GL.glGetShaderInfoLog, shader_objects.glGetInfoLogARB )

glGetShaderiv = alternate( GL.glGetShaderiv, shader_objects.glGetObjectParameterivARB )
glGetProgramiv = alternate( GL.glGetProgramiv, shader_objects.glGetObjectParameterivARB )

GL_VALIDATE_STATUS = GL.GL_VALIDATE_STATUS
GL_COMPILE_STATUS = GL.GL_COMPILE_STATUS
GL_LINK_STATUS = GL.GL_LINK_STATUS
GL_FALSE = GL.GL_FALSE
GL_TRUE = GL.GL_TRUE

class ShaderProgram( int ):
    """Integer sub-class with context-manager operation"""
    validated = False
    def __enter__( self ):
        """Start use of the program"""
        glUseProgram( self )
    def __exit__( self, typ, val, tb ):
        """Stop use of the program"""
        glUseProgram( 0 )
    
    def check_validate( self ):
        """Check that the program validates
        
        Validation has to occur *after* linking/loading
        
        raises ShaderValidationError on failures
        """
        glValidateProgram( self )
        validation = glGetProgramiv( self, GL_VALIDATE_STATUS )
        if validation == GL_FALSE:
            raise ShaderValidationError(
                """Validation failure (%r): %s"""%(
                validation,
                glGetProgramInfoLog( self ),
            ))
        self.validated = True
        return self

    def check_linked( self ):
        """Check link status for this program
        
        raises ShaderLinkError on failures
        """
        link_status = glGetProgramiv( self, GL_LINK_STATUS )
        if link_status == GL_FALSE:
            raise ShaderLinkError(
                """Link failure (%s): %s"""%(
                link_status,
                glGetProgramInfoLog( self ),
            ))
        return self

    def retrieve( self ):
        """Attempt to retrieve binary for this compiled shader
        
        Note that binaries for a program are *not* generally portable,
        they should be used solely for caching compiled programs for 
        local use; i.e. to reduce compilation overhead.
        
        returns (format,binaryData) for the shader program
        """
        from OpenGL.raw.GL._types import GLint,GLenum 
        from OpenGL.arrays import GLbyteArray
        size = GLint()
        glGetProgramiv( self, get_program_binary.GL_PROGRAM_BINARY_LENGTH, size )
        result = GLbyteArray.zeros( (size.value,))
        size2 = GLint()
        format = GLenum()
        get_program_binary.glGetProgramBinary( self, size.value, size2, format, result )
        return format.value, result 
    def load( self, format, binary, validate=True ):
        """Attempt to load binary-format for a pre-compiled shader
        
        See notes in retrieve
        """
        get_program_binary.glProgramBinary( self, format, binary, len(binary))
        if validate:
            self.check_validate()
        self.check_linked()
        return self

def compileProgram(*shaders, **named):
    """Create a new program, attach shaders and validate

    shaders -- arbitrary number of shaders to attach to the
        generated program.
    separable (keyword only) -- set the separable flag to allow 
        for partial installation of shader into the pipeline (see 
        glUseProgramStages)
    retrievable (keyword only) -- set the retrievable flag to 
        allow retrieval of the program binary representation, (see 
        glProgramBinary, glGetProgramBinary)
    validate (keyword only) -- if False, suppress automatic 
        validation against current GL state. In advanced usage 
        the validation can produce spurious errors. Note: this 
        function is *not* really intended for advanced usage,
        if you're finding yourself specifying this flag you 
        likely should be using your own shader management code.

    This convenience function is *not* standard OpenGL,
    but it does wind up being fairly useful for demos
    and the like.  You may wish to copy it to your code
    base to guard against PyOpenGL changes.

    Usage:

        shader = compileProgram(
            compileShader( source, GL_VERTEX_SHADER ),
            compileShader( source2, GL_FRAGMENT_SHADER ),
        )
        glUseProgram( shader )

    Note:
        If (and only if) validation of the linked program
        *passes* then the passed-in shader objects will be
        deleted from the GL.

    returns ShaderProgram() (GLuint) program reference
    raises RuntimeError subclasses {
        ShaderCompilationError, ShaderValidationError, ShaderLinkError,
    } when a link/validation failure occurs
    """
    program = glCreateProgram()
    if named.get('separable'):
        glProgramParameteri( program, separate_shader_objects.GL_PROGRAM_SEPARABLE, GL_TRUE )
    if named.get('retrievable'):
        glProgramParameteri( program, get_program_binary.GL_PROGRAM_BINARY_RETRIEVABLE_HINT, GL_TRUE )
    for shader in shaders:
        glAttachShader(program, shader)
    program = ShaderProgram( program )
    glLinkProgram(program)
    if named.get('validate', True):
        program.check_validate()
    program.check_linked()
    for shader in shaders:
        glDeleteShader(shader)
    return program
def compileShader( source, shaderType ):
    """Compile shader source of given type

    source -- GLSL source-code for the shader
    shaderType -- GLenum GL_VERTEX_SHADER, GL_FRAGMENT_SHADER, etc,

    returns GLuint compiled shader reference
    raises RuntimeError when a compilation failure occurs
    """
    if isinstance( source, (bytes,unicode)):
        source = [ source ]
    source = [ as_8_bit(s) for s in source ]
    shader = glCreateShader(shaderType)
    glShaderSource( shader, source )
    glCompileShader( shader )
    result = glGetShaderiv( shader, GL_COMPILE_STATUS )
    if not(result):
        # TODO: this will be wrong if the user has
        # disabled traditional unpacking array support.
        raise ShaderCompilationError(
            """Shader compile failure (%s): %s"""%(
                result,
                glGetShaderInfoLog( shader ),
            ),
            source,
            shaderType,
        )
    return shader

class ShaderCompilationError(RuntimeError):
    """Raised when a shader compilation fails"""
class ShaderValidationError(RuntimeError):
    """Raised when a program fails to validate"""
class ShaderLinkError(RuntimeError):
    """Raised when a shader link fails"""


'''OpenGL extension APPLE.texture_format_BGRA8888

This module customises the behaviour of the 
OpenGL.raw.GLES2.APPLE.texture_format_BGRA8888 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces BGRA_EXT as an acceptable external format.  
	This avoids byte swizzling when loading RGBA internal format
	textures, which may be stored in BGRA order internally.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/texture_format_BGRA8888.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.APPLE.texture_format_BGRA8888 import *
from OpenGL.raw.GLES2.APPLE.texture_format_BGRA8888 import _EXTENSION_NAME

def glInitTextureFormatBgra8888APPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_norm16

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_norm16 to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL ES 3.1 supports 8-bit (signed) normalized textures.
	
	This extension provides a set of new 16 bit signed normalized and
	unsigned normalized fixed point texture, renderbuffer and
	texture buffer formats.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_norm16.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_norm16 import *
from OpenGL.raw.GLES2.EXT.texture_norm16 import _EXTENSION_NAME

def glInitTextureNorm16EXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""ctypes-based OpenGL wrapper for Python

This is the PyOpenGL 3.x tree, it attempts to provide
a largely compatible API for code written with the
PyOpenGL 2.x series using the ctypes foreign function
interface system.

Configuration Variables:

There are a few configuration variables in this top-level
module.  Applications should be the only code that tweaks
these variables, mid-level libraries should not take it
upon themselves to disable/enable features at this level.
The implication there is that your library code should be
able to work with any of the valid configurations available
with these sets of flags.

Further, once any entry point has been loaded, the variables 
can no longer be updated.  The OpenGL._confligflags module 
imports the variables from this location, and once that 
import occurs the flags should no longer be changed.

    ERROR_CHECKING -- if set to a False value before
        importing any OpenGL.* libraries will completely
        disable error-checking.  This can dramatically
        improve performance, but makes debugging far
        harder.

        This is intended to be turned off *only* in a
        production environment where you *know* that
        your code is entirely free of situations where you
        use exception-handling to handle error conditions,
        i.e. where you are explicitly checking for errors
        everywhere they can occur in your code.

        Default: True

    ERROR_LOGGING -- If True, then wrap array-handler
        functions with  error-logging operations so that all exceptions
        will be reported to log objects in OpenGL.logs, note that
        this means you will get lots of error logging whenever you
        have code that tests by trying something and catching an
        error, this is intended to be turned on only during
        development so that you can see why something is failing.

        Errors are normally logged to the OpenGL.errors logger.

        Only triggers if ERROR_CHECKING is True

        Default: False

    ERROR_ON_COPY -- if set to a True value before
        importing the numpy/lists support modules, will
        cause array operations to raise
        OpenGL.error.CopyError if the operation
        would cause a data-copy in order to make the
        passed data-type match the target data-type.

        This effectively disables all list/tuple array
        support, as they are inherently copy-based.

        This feature allows for optimisation of your
        application.  It should only be enabled during
        testing stages to prevent raising errors on
        recoverable conditions at run-time.

        Default: False

    CONTEXT_CHECKING -- if set to True, PyOpenGL will wrap
        *every* GL and GLU call with a check to see if there
        is a valid context.  If there is no valid context
        then will throw OpenGL.errors.NoContext.  This is an
        *extremely* slow check and is not enabled by default,
        intended to be enabled in order to track down (wrong)
        code that uses GL/GLU entry points before the context
        has been initialized (something later Linux GLs are
        very picky about).

        Default: False

    STORE_POINTERS -- if set to True, PyOpenGL array operations
        will attempt to store references to pointers which are
        being passed in order to prevent memory-access failures
        if the pointed-to-object goes out of scope.  This
        behaviour is primarily intended to allow temporary arrays
        to be created without causing memory errors, thus it is
        trading off performance for safety.

        To use this flag effectively, you will want to first set
        ERROR_ON_COPY to True and eliminate all cases where you
        are copying arrays.  Copied arrays *will* segfault your
        application deep within the GL if you disable this feature!

        Once you have eliminated all copying of arrays in your
        application, you will further need to be sure that all
        arrays which are passed to the GL are stored for at least
        the time period for which they are active in the GL.  That
        is, you must be sure that your array objects live at least
        until they are no longer bound in the GL.  This is something
        you need to confirm by thinking about your application's
        structure.

        When you are sure your arrays won't cause seg-faults, you
        can set STORE_POINTERS=False in your application and enjoy
        a (slight) speed up.

        Note: this flag is *only* observed when ERROR_ON_COPY == True,
            as a safety measure to prevent pointless segfaults

        Default: True

    WARN_ON_FORMAT_UNAVAILABLE -- If True, generates
        logging-module warn-level events when a FormatHandler
        plugin is not loadable (with traceback).

        Default: False

    FULL_LOGGING -- If True, then wrap functions with
        logging operations which reports each call along with its
        arguments to  the OpenGL.calltrace logger at the INFO
        level.  This is *extremely* slow.  You should *not* enable
        this in production code!

        You will need to have a  logging configuration (e.g.
            logging.basicConfig()
        ) call  in your top-level script to see the results of the
        logging.

        Default: False

    ALLOW_NUMPY_SCALARS -- if True, we will wrap
        all GLint/GLfloat calls conversions with wrappers
        that allow for passing numpy scalar values.

        Note that this is experimental, *not* reliable,
        and very slow!

        Note that byte/char types are not wrapped.

        Default: False

    UNSIGNED_BYTE_IMAGES_AS_STRING -- if True, we will return
        GL_UNSIGNED_BYTE image-data as strings, instead of arrays
        for glReadPixels and glGetTexImage

        Default: True

    FORWARD_COMPATIBLE_ONLY -- only include OpenGL 3.1 compatible
        entry points.  Note that this will generally break most
        PyOpenGL code that hasn't been explicitly made "legacy free"
        via a significant rewrite.

        Default: False

    SIZE_1_ARRAY_UNPACK -- if True, unpack size-1 arrays to be
        scalar values, as done in PyOpenGL 1.5 -> 3.0.0, that is,
        if a glGenList( 1 ) is done, return a uint rather than
        an array of uints.

        Default: True

    USE_ACCELERATE -- if True, attempt to use the OpenGL_accelerate
        package to provide Cython-coded accelerators for core wrapping
        operations.

        Default: True
    
    MODULE_ANNOTATIONS -- if True, attempt to annotate alternates() and 
        constants to track in which module they are defined (only useful 
        for the documentation-generation passes, really).
        
        Default: False
    
    TYPE_ANNOTATIONS -- if True, set up type annotations in __annotations__
        on raw functions. This is mostly just so that people can play
        with the use of e.g. mypy or the like, but the values put in the
        annotations dictionary are generally either ctypes types or 
        ArrayDataType references, so this isn't *likely* to be all that useful
        without further work.
"""
from OpenGL.version import __version__
import os
def environ_key( name, default ):
    composed = 'PYOPENGL_%s'%name.upper()
    if composed in os.environ:
        value = os.environ[composed]
        if value.lower() in ('1','true'):
            return True 
        else:
            return False
    return os.environ.get( composed, default )

ERROR_CHECKING = environ_key( 'ERROR_CHECKING', True)
ERROR_LOGGING = environ_key( 'ERROR_LOGGING', False )
ERROR_ON_COPY = environ_key( 'ERROR_ON_COPY', False )
ARRAY_SIZE_CHECKING = environ_key( 'ARRAY_SIZE_CHECKING', True )
STORE_POINTERS = environ_key( 'STORE_POINTERS', True )
WARN_ON_FORMAT_UNAVAILABLE = False
FORWARD_COMPATIBLE_ONLY = False
SIZE_1_ARRAY_UNPACK = True
USE_ACCELERATE = environ_key( 'USE_ACCELERATE', True )
CONTEXT_CHECKING = environ_key( 'CONTEXT_CHECKING', False )

FULL_LOGGING = environ_key( 'FULL_LOGGING', False )
ALLOW_NUMPY_SCALARS = environ_key( 'ALLOW_NUMPY_SCALARS', False )
UNSIGNED_BYTE_IMAGES_AS_STRING = environ_key( 'UNSIGNED_BYTE_IMAGES_AS_STRING', True )
MODULE_ANNOTATIONS = False
TYPE_ANNOTATIONS = False


# Declarations of plugins provided by PyOpenGL itself
from OpenGL.plugins import PlatformPlugin, FormatHandler
PlatformPlugin( 'nt', 'OpenGL.platform.win32.Win32Platform' )
PlatformPlugin( 'darwin', 'OpenGL.platform.darwin.DarwinPlatform' )
PlatformPlugin( 'linux2', 'OpenGL.platform.glx.GLXPlatform' )
PlatformPlugin( 'linux', 'OpenGL.platform.glx.GLXPlatform' )
PlatformPlugin( 'posix', 'OpenGL.platform.glx.GLXPlatform' )
PlatformPlugin( 'x11', 'OpenGL.platform.glx.GLXPlatform' ) # xdg session type
PlatformPlugin( 'osmesa', 'OpenGL.platform.osmesa.OSMesaPlatform')
PlatformPlugin( 'egl', 'OpenGL.platform.egl.EGLPlatform')
PlatformPlugin( 'wayland', 'OpenGL.platform.egl.EGLPlatform') # xdg session type
PlatformPlugin( 'xwayland', 'OpenGL.platform.egl.EGLPlatform') # xdg session type, but use egl even though normally you'd expect GLX

import sys
if sys.version_info[0] < 3:
    # Python 3.x renames the built-in module
    _bi = '__builtin__'
else:
    _bi = 'builtins'

FormatHandler( 'none', 'OpenGL.arrays.nones.NoneHandler', [ _bi+'.NoneType'],isOutput=False )

if sys.version_info[0] < 3:
    FormatHandler( 'str', 'OpenGL.arrays.strings.StringHandler',[_bi+'.str'], isOutput=False )
    FormatHandler( 'unicode', 'OpenGL.arrays.strings.UnicodeHandler',[_bi+'.unicode'], isOutput=False )
else:
    FormatHandler( 'bytes', 'OpenGL.arrays.strings.StringHandler',[_bi+'.bytes'], isOutput=False )
    FormatHandler( 'str', 'OpenGL.arrays.strings.UnicodeHandler',[_bi+'.str'], isOutput=False )
    
FormatHandler( 'list', 'OpenGL.arrays.lists.ListHandler', [
    _bi+'.list',
    _bi+'.tuple',
], isOutput=False )
FormatHandler( 'numbers', 'OpenGL.arrays.numbers.NumberHandler', [
    _bi+'.int',
    _bi+'.float',
    _bi+'.long',
], isOutput=False )
FormatHandler(
    'ctypesarrays', 'OpenGL.arrays.ctypesarrays.CtypesArrayHandler',
    [
        '_ctypes.ArrayType',
        '_ctypes.PyCArrayType',
        '_ctypes.Array',
        '_ctypes.array.Array',
    ],
    isOutput=True,
)
FormatHandler(
    'ctypesparameter',
    'OpenGL.arrays.ctypesparameters.CtypesParameterHandler',
    [
        _bi+'.CArgObject',
        'ctypes.c_uint',
        'ctypes.c_int',
        'ctypes.c_float',
        'ctypes.c_double',
        'ctypes.c_ulong',
        'ctypes.c_long',
        'ctypes.c_longlong',
    ],
    isOutput=True,
)
FormatHandler( 'ctypespointer', 'OpenGL.arrays.ctypespointers.CtypesPointerHandler',[
    'ctypes.c_void_p',
    '_ctypes._Pointer',
    'ctypes.c_char_p',
    '_ctypes.pointer._Pointer',
],isOutput=False )
FormatHandler( 'numpy', 'OpenGL.arrays.numpymodule.NumpyHandler', [
    'numpy.ndarray',
    'numpy.core.memmap.memmap',
    'numpy.uint8',
    'numpy.uint16',
    'numpy.uint32',
    'numpy.uint64',
    'numpy.int8',
    'numpy.int16',
    'numpy.int32',
    'numpy.int64',
    'numpy.float16',
    'numpy.float32',
    'numpy.float64',
    'numpy.float128',
],isOutput=True )
FormatHandler( 'buffer', 'OpenGL.arrays.buffers.BufferHandler', [
    'OpenGL.arrays._buffers.Py_buffer',
    _bi+'.memoryview',
    _bi+'.bytearray',
],isOutput=True )
FormatHandler( 'vbo', 'OpenGL.arrays.vbo.VBOHandler', ['OpenGL.arrays.vbo.VBO','OpenGL_accelerate.vbo.VBO'],isOutput=False )
FormatHandler( 'vbooffset', 'OpenGL.arrays.vbo.VBOOffsetHandler', ['OpenGL.arrays.vbo.VBOOffset','OpenGL_accelerate.vbo.VBOOffset'],isOutput=False )



'''OpenGL extension OES.blend_subtract

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.blend_subtract to provide a more 
Python-friendly API

Overview (from the spec)
	
	Blending capability is extended by respecifying the entire blend
	equation.  While this document defines only two new equations, the
	BlendEquationOES procedure that it defines will be used by subsequent
	extensions to define additional blending equations.
	
	In addition to the default blending equation, two new blending equations
	are specified.  These equations are similar to the default blending 
	equation, but produce the difference of its left and right hand sides, 
	rather than the sum.  Image differences are useful in many image 
	processing applications.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/blend_subtract.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.blend_subtract import *
from OpenGL.raw.GLES1.OES.blend_subtract import _EXTENSION_NAME

def glInitBlendSubtractOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.gl_spirv

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.gl_spirv to provide a more 
Python-friendly API

Overview (from the spec)
	
	This is version 100 of the GL_ARB_gl_spirv extension.
	
	This extension does two things:
	
	1. Allows a SPIR-V module to be specified as containing a programmable
	   shader stage, rather than using GLSL, whatever the source language
	   was used to create the SPIR-V module.
	
	2. Modifies GLSL to be a source language for creating SPIR-V modules
	   for OpenGL consumption. Such GLSL can be used to create such SPIR-V
	   modules, outside of the OpenGL runtime.
	
	Enabling GLSL SPIR-V Features
	-----------------------------
	
	This extension is not enabled with a #extension as other extensions are.
	It is also not enabled through use of a profile or #version.  The intended
	level of GLSL features comes from the traditional use of #version, profile,
	and #extension.
	
	Instead, use of this extension is an effect of using a GLSL front-end in a
	mode that has it generate SPIR-V for OpenGL.  Such tool use is outside the
	scope of using the OpenGL API and outside the definition of GLSL and this
	extension. See the documentation of the compiler to see how to request
	generation of SPIR-V for OpenGL.
	
	When a front-end is used to accept the GLSL features in this extension, it
	must error check and reject shaders not adhering to this specification, and
	accept those that do.  Implementation-dependent maximums and capabilities
	are supplied to, or part of, the front-end, so it can do error checking
	against them.
	
	A shader can query the level of GLSL SPIR-V support available, using the
	predefined
	
	    #define GL_SPIRV 100
	
	This allows shader code to say, for example,
	
	    #ifdef GL_SPIRV
	        layout(constant_id = 11) const int count = 4;
	        #if GL_SPIRV > 100
	            ...
	        #endif
	    #else
	        const int count = 6;
	    #endif
	
	SPIR-V Modules
	--------------
	
	An entire stage is held in a single SPIR-V module.  The SPIR-V model is
	that multiple (e.g., GLSL) compilation units forming a single stage
	in a high-level language are all compiled and linked together to form a
	single entry point (and its call tree) in a SPIR-V module.  This would be
	done prior to using the OpenGL API to create a program object containing the
	stage.
	
	The OpenGL API expects the SPIR-V module to have already been validated,
	and can return an error if it discovers anything invalid in
	the module.  An invalid SPIR-V module is allowed to result in undefined
	behavior.
	
	Specialization Constants
	------------------------
	
	SPIR-V specialization constants, which can be set later by the client API,
	can be declared using "layout(constant_id=...)". For example, to make a
	specialization constant with a default value of 12:
	
	    layout(constant_id = 17) const int arraySize = 12;
	
	Above, "17" is the ID by which the API or other tools can later refer to
	this specific specialization constant.  The API or an intermediate tool can
	then change its value to another constant integer before it is fully
	lowered to executable code.  If it is never changed before final lowering,
	it will retain the value of 12.
	
	Specialization constants have const semantics, except they don't fold.
	Hence, an array can be declared with 'arraySize' from above:
	
	    vec4 data[arraySize];  // legal, even though arraySize might change
	
	Specialization constants can be in expressions:
	
	    vec4 data2[arraySize + 2];
	
	This will make data2 be sized by 2 more than whatever constant value
	'arraySize' has when it is time to lower the shader to executable code.
	
	An expression formed with specialization constants also behaves in the
	shader like a specialization constant, not a like a constant.
	
	    arraySize + 2       // a specialization constant (with no constant_id)
	
	Such expressions can be used in the same places as a constant.
	
	The constant_id can only be applied to a scalar *int*, a scalar *float*
	or a scalar *bool*.
	
	Only basic operators and constructors can be applied to a specialization
	constant and still result in a specialization constant:
	
	    layout(constant_id = 17) const int arraySize = 12;
	    sin(float(arraySize));    // result is not a specialization constant
	
	While SPIR-V specialization constants are only for scalars, a vector
	can be made by operations on scalars:
	
	    layout(constant_id = 18) const int scX = 1;
	    layout(constant_id = 19) const int scZ = 1;
	    const vec3 scVec = vec3(scX, 1, scZ);  // partially specialized vector
	
	A built-in variable can have a 'constant_id' attached to it:
	
	    layout(constant_id = 18) gl_MaxImageUnits;
	
	This makes it behave as a specialization constant.  It is not a full
	redeclaration; all other characteristics are left intact from the
	original built-in declaration.
	
	The built-in vector gl_WorkGroupSize can be specialized using special
	layout local_size_{xyz}_id's applied to the "in" qualifier.  For example:
	
	    layout(local_size_x_id = 18, local_size_z_id = 19) in;
	
	This leaves gl_WorkGroupSize.y as a non-specialization constant, with
	gl_WorkGroupSize being a partially specialized vector.  Its x and z
	components can be later specialized using the ID's 18 and 19.
	
	gl_FragColor
	------------
	
	The fragment-stage built-in gl_FragColor, which implies a broadcast to all
	outputs, is not present in SPIR-V. Shaders where writing to gl_FragColor
	is allowed can still write to it, but it only means to write to an output:
	 - of the same type as gl_FragColor
	 - decorated with location 0
	 - not decorated as a built-in variable.
	There is no implicit broadcast.
	
	Mapping to SPIR-V
	-----------------
	
	For informational purposes (non-specification), the following is an
	expected way for an implementation to map GLSL constructs to SPIR-V
	constructs:
	
	Mapping of Storage Classes:
	
	  uniform sampler2D...;        -> UniformConstant
	  uniform variable (non-block) -> UniformConstant
	  uniform blockN { ... } ...;  -> Uniform, with Block decoration
	  in / out variable            -> Input/Output, possibly with block (below)
	  in / out block...            -> Input/Output, with Block decoration
	  buffer  blockN { ... } ...;  -> Uniform, with BufferBlock decoration
	  ... uniform atomic_uint ...  -> AtomicCounter
	  shared                       -> Workgroup
	  <normal global>              -> Private
	
	Mapping of input/output blocks or variables is the same for all versions
	of GLSL. To the extent variables or members are available in a
	version and a stage, its location is as follows:
	
	  These are mapped to SPIR-V individual variables, with similarly spelled
	  built-in decorations (except as noted):
	
	    General:
	
	        in gl_VertexID
	        in gl_InstanceID
	        in gl_InvocationID
	        in gl_PatchVerticesIn      (PatchVertices)
	        in gl_PrimitiveIDIn        (PrimitiveID)
	        in/out gl_PrimitiveID      (in/out based only on storage qualifier)
	        in gl_TessCoord
	
	        in/out gl_Layer
	        in/out gl_ViewportIndex
	
	        patch in/out gl_TessLevelOuter  (uses Patch decoration)
	        patch in/out gl_TessLevelInner  (uses Patch decoration)
	
	    Compute stage only:
	
	        in gl_NumWorkGroups
	        in gl_WorkGroupSize
	        in gl_WorkGroupID
	        in gl_LocalInvocationID
	        in gl_GlobalInvocationID
	        in gl_LocalInvocationIndex
	
	    Fragment stage only:
	
	        in gl_FragCoord
	        in gl_FrontFacing
	        in gl_ClipDistance
	        in gl_CullDistance
	        in gl_PointCoord
	        in gl_SampleID
	        in gl_SamplePosition
	        in gl_HelperInvocation
	        out gl_FragDepth
	        in gl_SampleMaskIn        (SampleMask)
	        out gl_SampleMask         (in/out based only on storage qualifier)
	
	  These are mapped to SPIR-V blocks, as implied by the pseudo code, with
	  the members decorated with similarly spelled built-in decorations:
	
	    Non-fragment stage:
	
	        in/out gl_PerVertex {  // some subset of these members will be used
	            gl_Position
	            gl_PointSize
	            gl_ClipDistance
	            gl_CullDistance
	        }                      // name of block is for debug only
	
	  There is at most one input and one output block per stage in SPIR-V.
	  The subset and order of members will match between stages sharing an
	  interface.
	
	Mapping of precise:
	
	  precise -> NoContraction
	
	Mapping of atomic_uint /offset/ layout qualifier
	
	  offset         ->  Offset (decoration)
	
	Mapping of images
	
	  imageLoad()   -> OpImageRead
	  imageStore()  -> OpImageWrite
	  texelFetch()  -> OpImageFetch
	
	  imageAtomicXXX(params, data)  -> %ptr = OpImageTexelPointer params
	                                          OpAtomicXXX %ptr, data
	
	  XXXQueryXXX(combined) -> %image = OpImage combined
	                                    OpXXXQueryXXX %image
	
	Mapping of layouts
	
	  std140/std430  ->  explicit *Offset*, *ArrayStride*, and *MatrixStride*
	                     Decoration on struct members
	  shared/packed  ->  not allowed
	  <default>      ->  not shared, but std140 or std430
	  xfb_offset     ->  *Offset* Decoration on the object or struct member
	  xfb_buffer     ->  *XfbBuffer* Decoration on the object
	  xfb_stride     ->  *XfbStride* Decoration on the object
	  any xfb_*      ->  the *Xfb* Execution Mode is set
	  captured XFB   ->  has both *XfbBuffer* and *Offset*
	  non-captured   ->  lacking *XfbBuffer* or *Offset*
	
	  max_vertices   ->  OutputVertices
	
	Mapping of barriers
	
	  barrier() (compute) -> OpControlBarrier(/*Execution*/Workgroup,
	                                          /*Memory*/Workgroup,
	                                          /*Semantics*/AcquireRelease |
	                                                       WorkgroupMemory)
	
	  barrier() (tess control) -> OpControlBarrier(/*Execution*/Workgroup,
	                                               /*Memory*/Invocation,
	                                               /*Semantics*/None)
	
	  memoryBarrier() -> OpMemoryBarrier(/*Memory*/Device,
	                                     /*Semantics*/AcquireRelease |
	                                                  UniformMemory |
	                                                  WorkgroupMemory |
	                                                  ImageMemory)
	
	  memoryBarrierBuffer() -> OpMemoryBarrier(/*Memory*/Device,
	                                           /*Semantics*/AcquireRelease |
	                                                        UniformMemory)
	
	  memoryBarrierShared() -> OpMemoryBarrier(/*Memory*/Device,
	                                           /*Semantics*/AcquireRelease |
	                                                        WorkgroupMemory)
	
	  memoryBarrierImage() -> OpMemoryBarrier(/*Memory*/Device,
	                                          /*Semantics*/AcquireRelease |
	                                                       ImageMemory)
	
	  groupMemoryBarrier() -> OpMemoryBarrier(/*Memory*/Workgroup,
	                                          /*Semantics*/AcquireRelease |
	                                                       UniformMemory |
	                                                       WorkgroupMemory |
	                                                       ImageMemory)
	
	Mapping of atomics
	
	  all atomic builtin functions -> Semantics = None(Relaxed)
	
	  atomicExchange()      -> OpAtomicExchange
	  imageAtomicExchange() -> OpAtomicExchange
	  atomicCompSwap()      -> OpAtomicCompareExchange
	  imageAtomicCompSwap() -> OpAtomicCompareExchange
	  NA                    -> OpAtomicCompareExchangeWeak
	
	  atomicCounterIncrement -> OpAtomicIIncrement
	  atomicCounterDecrement -> OpAtomicIDecrement (with post decrement)
	  atomicCounter          -> OpAtomicLoad
	
	Mapping of uniform initializers
	
	  Using the OpVariable initializer logic, but only from a constant
	  instruction (not a global one).
	
	Mapping of other instructions
	
	  %     -> OpUMod/OpSMod
	  mod() -> OpFMod
	  NA    -> OpSRem/OpFRem
	
	  pack/unpack (conversion)    -> pack/unpack in GLSL extended instructions
	  pack/unpack (no conversion) -> OpBitcast
	
	Differences Relative to Other Specifications
	--------------------------------------------
	
	The following summarizes the set of differences to existing specifications.
	
	Additional use of existing SPIR-V features, relative to Vulkan:
	  + The *UniformConstant* Storage Class can be used on individual
	    variables at global scope. (That is, uniforms don't have to be in a
	    block, unless they are built-in members that are in block in GLSL
	    version 4.5.)
	  + *AtomicCounter* Storage Class can use the *Offset* decoration
	  + OriginLowerLeft
	  + Uniforms support constant initializers.
	
	Corresponding features that GLSL keeps, despite GL_KHR_vulkan_glsl removal:
	  . default uniforms (those not inside a uniform block)
	  . atomic-counter bindings have the /offset/ layout qualifier
	  . special rules for locations for input doubles in the vertex shader
	  . origin_lower_left
	
	Addition of features to GLSL:
	  + specialization constants, as per GL_KHR_vulkan_glsl
	  + #define GL_SPIRV 100, when compiled for OpenGL and SPIR-V generation
	  + offset can organize members in a different order than declaration order
	
	Non-acceptance of SPIR-V features, relative to Vulkan:
	  - VertexIndex and InstanceIndex built-in decorations cannot be used
	  - Push-constant buffers cannot be used
	  - *DescriptorSet* must always be 0, if present
	  - input targets and input attachments
	  - OpTypeSampler
	
	Corresponding features not added to GLSL that GL_KHR_vulkan_glsl added:
	  . gl_VertexIndex and gl_InstanceIndex
	      (gl_VertexID and gl_InstanceID have same semantics as in GLSL)
	  . push_constant buffers
	  . descriptor sets
	  . input_attachment_index and accessing input targets
	  . shader-combining of textures and samplers
	
	Removal of features from GLSL, as removed by GL_KHR_vulkan_glsl:
	  - subroutines
	  - the already deprecated texturing functions (e.g., texture2D())
	  - the already deprecated noise functions (e.g., noise1())
	  - compatibility profile features
	  - gl_DepthRangeParameters and gl_NumSamples
	  - *shared* and *packed* block layouts
	
	Addition of features to The OpenGL Graphics System:
	  + a command to associate a SPIR-V module with a program (ShaderBinary)
	  + a command to select a SPIR-V entry point and set specialization
	    constants in a SPIR-V module (SpecializeShaderARB)
	  + a new appendix for SPIR-V validation rules, precision, etc.
	
	Changes of system features, relative to Vulkan:
	  . Vulkan uses only one binding point for a resource array, while OpenGL
	    still uses multiple binding points, so binding numbers are counted
	    differently for SPIR-V used in Vulkan and OpenGL
	  . gl_FragColor can be written to, but it won't broadcast, for versions of
	    OpenGL that support gl_FragColor
	  . Vulkan does not allow multi-dimensional arrays of resources like
	    UBOs and SSBOs in its SPIR-V environment spec. SPIR-V supports
	    it and OpenGL already allows this for GLSL shaders. SPIR-V
	    for OpenGL also allows it.
	
	Additions to the SPIR-V specification:
	  + *Offset* can also apply to an object, for transform feedback.
	  + *Offset* can also apply to a default uniform, for atomic_uint offset.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/gl_spirv.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.gl_spirv import *
from OpenGL.raw.GL.ARB.gl_spirv import _EXTENSION_NAME

def glInitGlSpirvARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.multi_draw_indirect

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.multi_draw_indirect to provide a more 
Python-friendly API

Overview (from the spec)
	
	The ARB_draw_indirect extension (included in OpenGL 4.0 and OpenGL ES 3.1)
	introduced mechanisms whereby the parameters for a draw function may be
	provided in a structure contained in a buffer object rather than as
	parameters to the drawing procedure. This is known as an indirect draw and
	is exposed as two new functions, glDrawArraysIndirect and
	glDrawElementsIndirect. Each of these functions generates a single batch
	of primitives.
	
	This extension builds on this functionality by providing procedures to
	invoke multiple draws from a single procedure call. This allows large
	batches of drawing commands to be assembled in server memory (via a buffer
	object) which may then be dispatched through a single function call.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/multi_draw_indirect.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.multi_draw_indirect import *
from OpenGL.raw.GLES2.EXT.multi_draw_indirect import _EXTENSION_NAME

def glInitMultiDrawIndirectEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glMultiDrawArraysIndirectEXT.indirect size not checked against 'drawcount,stride'
glMultiDrawArraysIndirectEXT=wrapper.wrapper(glMultiDrawArraysIndirectEXT).setInputArraySize(
    'indirect', None
)
# INPUT glMultiDrawElementsIndirectEXT.indirect size not checked against 'drawcount,stride'
glMultiDrawElementsIndirectEXT=wrapper.wrapper(glMultiDrawElementsIndirectEXT).setInputArraySize(
    'indirect', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension ATI.separate_stencil

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.separate_stencil to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides the ability to modify the stencil buffer
	differently based on the facing direction of the primitive that
	generated the fragment.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/separate_stencil.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ATI.separate_stencil import *
from OpenGL.raw.GL.ATI.separate_stencil import _EXTENSION_NAME

def glInitSeparateStencilATI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_mesh_shader'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_mesh_shader',error_checker=_errors._error_checker)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_MESH_SHADER_NV=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_MESH_SHADER_NV',0x959E)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TASK_SHADER_NV=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TASK_SHADER_NV',0x959F)
GL_MAX_COMBINED_MESH_UNIFORM_COMPONENTS_NV=_C('GL_MAX_COMBINED_MESH_UNIFORM_COMPONENTS_NV',0x8E67)
GL_MAX_COMBINED_TASK_UNIFORM_COMPONENTS_NV=_C('GL_MAX_COMBINED_TASK_UNIFORM_COMPONENTS_NV',0x8E6F)
GL_MAX_DRAW_MESH_TASKS_COUNT_NV=_C('GL_MAX_DRAW_MESH_TASKS_COUNT_NV',0x953D)
GL_MAX_MESH_ATOMIC_COUNTERS_NV=_C('GL_MAX_MESH_ATOMIC_COUNTERS_NV',0x8E65)
GL_MAX_MESH_ATOMIC_COUNTER_BUFFERS_NV=_C('GL_MAX_MESH_ATOMIC_COUNTER_BUFFERS_NV',0x8E64)
GL_MAX_MESH_IMAGE_UNIFORMS_NV=_C('GL_MAX_MESH_IMAGE_UNIFORMS_NV',0x8E62)
GL_MAX_MESH_OUTPUT_PRIMITIVES_NV=_C('GL_MAX_MESH_OUTPUT_PRIMITIVES_NV',0x9539)
GL_MAX_MESH_OUTPUT_VERTICES_NV=_C('GL_MAX_MESH_OUTPUT_VERTICES_NV',0x9538)
GL_MAX_MESH_SHADER_STORAGE_BLOCKS_NV=_C('GL_MAX_MESH_SHADER_STORAGE_BLOCKS_NV',0x8E66)
GL_MAX_MESH_TEXTURE_IMAGE_UNITS_NV=_C('GL_MAX_MESH_TEXTURE_IMAGE_UNITS_NV',0x8E61)
GL_MAX_MESH_TOTAL_MEMORY_SIZE_NV=_C('GL_MAX_MESH_TOTAL_MEMORY_SIZE_NV',0x9536)
GL_MAX_MESH_UNIFORM_BLOCKS_NV=_C('GL_MAX_MESH_UNIFORM_BLOCKS_NV',0x8E60)
GL_MAX_MESH_UNIFORM_COMPONENTS_NV=_C('GL_MAX_MESH_UNIFORM_COMPONENTS_NV',0x8E63)
GL_MAX_MESH_VIEWS_NV=_C('GL_MAX_MESH_VIEWS_NV',0x9557)
GL_MAX_MESH_WORK_GROUP_INVOCATIONS_NV=_C('GL_MAX_MESH_WORK_GROUP_INVOCATIONS_NV',0x95A2)
GL_MAX_MESH_WORK_GROUP_SIZE_NV=_C('GL_MAX_MESH_WORK_GROUP_SIZE_NV',0x953B)
GL_MAX_TASK_ATOMIC_COUNTERS_NV=_C('GL_MAX_TASK_ATOMIC_COUNTERS_NV',0x8E6D)
GL_MAX_TASK_ATOMIC_COUNTER_BUFFERS_NV=_C('GL_MAX_TASK_ATOMIC_COUNTER_BUFFERS_NV',0x8E6C)
GL_MAX_TASK_IMAGE_UNIFORMS_NV=_C('GL_MAX_TASK_IMAGE_UNIFORMS_NV',0x8E6A)
GL_MAX_TASK_OUTPUT_COUNT_NV=_C('GL_MAX_TASK_OUTPUT_COUNT_NV',0x953A)
GL_MAX_TASK_SHADER_STORAGE_BLOCKS_NV=_C('GL_MAX_TASK_SHADER_STORAGE_BLOCKS_NV',0x8E6E)
GL_MAX_TASK_TEXTURE_IMAGE_UNITS_NV=_C('GL_MAX_TASK_TEXTURE_IMAGE_UNITS_NV',0x8E69)
GL_MAX_TASK_TOTAL_MEMORY_SIZE_NV=_C('GL_MAX_TASK_TOTAL_MEMORY_SIZE_NV',0x9537)
GL_MAX_TASK_UNIFORM_BLOCKS_NV=_C('GL_MAX_TASK_UNIFORM_BLOCKS_NV',0x8E68)
GL_MAX_TASK_UNIFORM_COMPONENTS_NV=_C('GL_MAX_TASK_UNIFORM_COMPONENTS_NV',0x8E6B)
GL_MAX_TASK_WORK_GROUP_INVOCATIONS_NV=_C('GL_MAX_TASK_WORK_GROUP_INVOCATIONS_NV',0x95A3)
GL_MAX_TASK_WORK_GROUP_SIZE_NV=_C('GL_MAX_TASK_WORK_GROUP_SIZE_NV',0x953C)
GL_MESH_OUTPUT_PER_PRIMITIVE_GRANULARITY_NV=_C('GL_MESH_OUTPUT_PER_PRIMITIVE_GRANULARITY_NV',0x9543)
GL_MESH_OUTPUT_PER_VERTEX_GRANULARITY_NV=_C('GL_MESH_OUTPUT_PER_VERTEX_GRANULARITY_NV',0x92DF)
GL_MESH_OUTPUT_TYPE_NV=_C('GL_MESH_OUTPUT_TYPE_NV',0x957B)
GL_MESH_PRIMITIVES_OUT_NV=_C('GL_MESH_PRIMITIVES_OUT_NV',0x957A)
GL_MESH_SHADER_BIT_NV=_C('GL_MESH_SHADER_BIT_NV',0x00000040)
GL_MESH_SHADER_NV=_C('GL_MESH_SHADER_NV',0x9559)
GL_MESH_SUBROUTINE_NV=_C('GL_MESH_SUBROUTINE_NV',0x957C)
GL_MESH_SUBROUTINE_UNIFORM_NV=_C('GL_MESH_SUBROUTINE_UNIFORM_NV',0x957E)
GL_MESH_VERTICES_OUT_NV=_C('GL_MESH_VERTICES_OUT_NV',0x9579)
GL_MESH_WORK_GROUP_SIZE_NV=_C('GL_MESH_WORK_GROUP_SIZE_NV',0x953E)
GL_REFERENCED_BY_MESH_SHADER_NV=_C('GL_REFERENCED_BY_MESH_SHADER_NV',0x95A0)
GL_REFERENCED_BY_TASK_SHADER_NV=_C('GL_REFERENCED_BY_TASK_SHADER_NV',0x95A1)
GL_TASK_SHADER_BIT_NV=_C('GL_TASK_SHADER_BIT_NV',0x00000080)
GL_TASK_SHADER_NV=_C('GL_TASK_SHADER_NV',0x955A)
GL_TASK_SUBROUTINE_NV=_C('GL_TASK_SUBROUTINE_NV',0x957D)
GL_TASK_SUBROUTINE_UNIFORM_NV=_C('GL_TASK_SUBROUTINE_UNIFORM_NV',0x957F)
GL_TASK_WORK_GROUP_SIZE_NV=_C('GL_TASK_WORK_GROUP_SIZE_NV',0x953F)
GL_UNIFORM_BLOCK_REFERENCED_BY_MESH_SHADER_NV=_C('GL_UNIFORM_BLOCK_REFERENCED_BY_MESH_SHADER_NV',0x959C)
GL_UNIFORM_BLOCK_REFERENCED_BY_TASK_SHADER_NV=_C('GL_UNIFORM_BLOCK_REFERENCED_BY_TASK_SHADER_NV',0x959D)
@_f
@_p.types(None,_cs.GLintptr)
def glDrawMeshTasksIndirectNV(indirect):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint)
def glDrawMeshTasksNV(first,count):pass
@_f
@_p.types(None,_cs.GLintptr,_cs.GLintptr,_cs.GLsizei,_cs.GLsizei)
def glMultiDrawMeshTasksIndirectCountNV(indirect,drawcount,maxdrawcount,stride):pass
@_f
@_p.types(None,_cs.GLintptr,_cs.GLsizei,_cs.GLsizei)
def glMultiDrawMeshTasksIndirectNV(indirect,drawcount,stride):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_QCOM_framebuffer_foveated'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_QCOM_framebuffer_foveated',error_checker=_errors._error_checker)
GL_FOVEATION_ENABLE_BIT_QCOM=_C('GL_FOVEATION_ENABLE_BIT_QCOM',0x00000001)
GL_FOVEATION_SCALED_BIN_METHOD_BIT_QCOM=_C('GL_FOVEATION_SCALED_BIN_METHOD_BIT_QCOM',0x00000002)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint,arrays.GLuintArray)
def glFramebufferFoveationConfigQCOM(framebuffer,numLayers,focalPointsPerLayer,requestedFeatures,providedFeatures):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glFramebufferFoveationParametersQCOM(framebuffer,layer,focalPoint,focalX,focalY,gainX,gainY,foveaArea):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIX_async_pixel'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIX_async_pixel',error_checker=_errors._error_checker)
GL_ASYNC_DRAW_PIXELS_SGIX=_C('GL_ASYNC_DRAW_PIXELS_SGIX',0x835D)
GL_ASYNC_READ_PIXELS_SGIX=_C('GL_ASYNC_READ_PIXELS_SGIX',0x835E)
GL_ASYNC_TEX_IMAGE_SGIX=_C('GL_ASYNC_TEX_IMAGE_SGIX',0x835C)
GL_MAX_ASYNC_DRAW_PIXELS_SGIX=_C('GL_MAX_ASYNC_DRAW_PIXELS_SGIX',0x8360)
GL_MAX_ASYNC_READ_PIXELS_SGIX=_C('GL_MAX_ASYNC_READ_PIXELS_SGIX',0x8361)
GL_MAX_ASYNC_TEX_IMAGE_SGIX=_C('GL_MAX_ASYNC_TEX_IMAGE_SGIX',0x835F)




'''OpenGL extension ANGLE.pack_reverse_row_order

This module customises the behaviour of the 
OpenGL.raw.GLES2.ANGLE.pack_reverse_row_order to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces a mechanism to allow reversing the order
	in which image rows are written into a pack destination. This
	effectively allows an application to flip the results of a ReadPixels
	in the y direction operation without having to render upside down.
	
	The coordinate system of OpenGL is vertically reversed in comparison to a
	number of other graphics systems such as native windowing APIs. Applications
	that perform ReadPixels may have to either render to an intermediate color
	buffer before calling ReadPixels or perform a flip in software after
	ReadPixels. In some systems the GL can perform the row reversal during
	ReadPixels without incurring additional cost.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ANGLE/pack_reverse_row_order.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ANGLE.pack_reverse_row_order import *
from OpenGL.raw.GLES2.ANGLE.pack_reverse_row_order import _EXTENSION_NAME

def glInitPackReverseRowOrderANGLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NVX_gpu_multicast2'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NVX_gpu_multicast2',error_checker=_errors._error_checker)
GL_UPLOAD_GPU_MASK_NVX=_C('GL_UPLOAD_GPU_MASK_NVX',0x954A)
@_f
@_p.types(_cs.GLuint,_cs.GLsizei,arrays.GLuintArray,arrays.GLuint64Array,_cs.GLuint,_cs.GLbitfield,_cs.GLuint,_cs.GLuint,_cs.GLintptr,_cs.GLintptr,_cs.GLsizeiptr,_cs.GLsizei,arrays.GLuintArray,arrays.GLuint64Array)
def glAsyncCopyBufferSubDataNVX(waitSemaphoreCount,waitSemaphoreArray,fenceValueArray,readGpu,writeGpuMask,readBuffer,writeBuffer,readOffset,writeOffset,size,signalSemaphoreCount,signalSemaphoreArray,signalValueArray):pass
@_f
@_p.types(_cs.GLuint,_cs.GLsizei,arrays.GLuintArray,arrays.GLuint64Array,_cs.GLuint,_cs.GLbitfield,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,arrays.GLuintArray,arrays.GLuint64Array)
def glAsyncCopyImageSubDataNVX(waitSemaphoreCount,waitSemaphoreArray,waitValueArray,srcGpu,dstGpuMask,srcName,srcTarget,srcLevel,srcX,srcY,srcZ,dstName,dstTarget,dstLevel,dstX,dstY,dstZ,srcWidth,srcHeight,srcDepth,signalSemaphoreCount,signalSemaphoreArray,signalValueArray):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLintArray)
def glMulticastScissorArrayvNVX(gpu,first,count,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLfloatArray)
def glMulticastViewportArrayvNVX(gpu,first,count,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLfloat,_cs.GLfloat)
def glMulticastViewportPositionWScaleNVX(gpu,index,xcoeff,ycoeff):pass
@_f
@_p.types(None,_cs.GLbitfield)
def glUploadGpuMaskNVX(mask):pass



'''OpenGL extension NV.fragment_shader_barycentric

This module customises the behaviour of the 
OpenGL.raw.GL.NV.fragment_shader_barycentric to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension advertises OpenGL support for the OpenGL Shading Language
	(GLSL) extension "NV_fragment_shader_barycentric", which provides fragment
	shader built-in variables holding barycentric weight vectors that identify
	the location of the fragment within its primitive.  Additionally, the GLSL
	extension allows fragment the ability to read raw attribute values for
	each of the vertices of the primitive that produced the fragment.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/fragment_shader_barycentric.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.fragment_shader_barycentric import *
from OpenGL.raw.GL.NV.fragment_shader_barycentric import _EXTENSION_NAME

def glInitFragmentShaderBarycentricNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension MESA.ycbcr_texture

This module customises the behaviour of the 
OpenGL.raw.GL.MESA.ycbcr_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension supports texture images stored in the YCbCr format.
	There is no support for converting YCbCr images to RGB or vice versa
	during pixel transfer.  The texture's YCbCr colors are converted to
	RGB during texture sampling, after-which, all the usual per-fragment
	operations take place.  Only 2D texture images are supported (not
	glDrawPixels, glReadPixels, etc).
	
	A YCbCr pixel (texel) is a 16-bit unsigned short with two components.
	The first component is luminance (Y).  For pixels in even-numbered
	image columns, the second component is Cb.  For pixels in odd-numbered
	image columns, the second component is Cr.  If one were to convert the
	data to RGB one would need to examine two pixels from columns N and N+1
	(where N is even) to deduce the RGB color.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/ycbcr_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.MESA.ycbcr_texture import *
from OpenGL.raw.GL.MESA.ycbcr_texture import _EXTENSION_NAME

def glInitYcbcrTextureMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.draw_vulkan_image

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.draw_vulkan_image to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a new function, DrawVkImageNV(), allowing
	applications to draw a screen-aligned rectangle displaying some or all of
	the contents of a two-dimensional Vulkan VkImage.  Callers specify a
	Vulkan VkImage handle, an optional OpenGL sampler object, window
	coordinates of the rectangle to draw, and texture coordinates corresponding
	to the corners of the rectangle.  For each fragment produced by the
	rectangle, DrawVkImageNV  interpolates the texture coordinates, performs
	a texture lookup, and uses the texture result as the fragment color.
	
	No shaders are used by DrawVkImageNV; the results of the texture lookup
	are used in lieu of a fragment shader output.  The fragments generated are
	processed by all per-fragment operations.  In particular,
	DrawVkImageNV() fully supports blending and multisampling.
	
	In order to synchronize between Vulkan and OpenGL there are three other
	functions provided; WaitVkSemaphoreNV(), SignalVkSemaphoreNV() and
	SignalVkFenceNV().  These allow OpenGL to wait for Vulkan to complete work
	and also Vulkan to wait for OpenGL to complete work.  Together OpenGL
	and Vulkan can synchronize on the server without application
	interation.
	
	Finally the function GetVkProcAddrNV() is provided to allow the OpenGL
	context to query the Vulkan entry points directly and avoid having to
	load them through the typical Vulkan loader.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/draw_vulkan_image.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.draw_vulkan_image import *
from OpenGL.raw.GLES2.NV.draw_vulkan_image import _EXTENSION_NAME

def glInitDrawVulkanImageNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGetVkProcAddrNV.name size not checked against 'name'
glGetVkProcAddrNV=wrapper.wrapper(glGetVkProcAddrNV).setInputArraySize(
    'name', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension OES.EGL_image_external_essl3

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.EGL_image_external_essl3 to provide a more 
Python-friendly API

Overview (from the spec)
	
	OES_EGL_image_external provides a mechanism for creating EGLImage texture
	targets from EGLImages, but only specified language interactions for the
	OpenGL ES Shading Language version 1.0. This extension adds support for
	versions 3.x of the OpenGL ES Shading Language.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/EGL_image_external_essl3.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.EGL_image_external_essl3 import *
from OpenGL.raw.GLES2.OES.EGL_image_external_essl3 import _EXTENSION_NAME

def glInitEglImageExternalEssl3OES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.clip_control

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.clip_control to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides additional clip control modes to configure how
	clip space is mapped to window space.  This extension's goal is to 1)
	allow OpenGL to effectively match Direct3D's coordinate system
	conventions, and 2) potentially improve the numerical precision of the Z
	coordinate mapping.
	
	Developers interested in this functionality may be porting content
	from Direct3D to OpenGL and/or interested in improving the numerical
	accuracy of depth testing, particularly with floating-point depth
	buffers.
	
	OpenGL's initial and conventional clip control state is configured by:
	
	    glClipControl(GL_LOWER_LEFT, GL_NEGATIVE_ONE_TO_ONE);
	
	where geometry with (x,y) normalized device coordinates of (-1,-1)
	correspond to the lower-left corner of the viewport and the near and far
	planes correspond to z normalized device coordinates of -1 and +1,
	respectively.
	
	This extension can be used to render content used in a Direct3D
	application in OpenGL in a straightforward way without modifying vertex or
	matrix data.  When rendering into a window, the command
	
	    glClipControl(GL_LOWER_LEFT, GL_ZERO_TO_ONE);
	
	configures the near clip plane to correspond to a z normalized device
	coordinate of 0 as in Direct3D.  Geometry with (x,y) normalized device
	coordinates of (-1,-1) correspond to the lower-left corner of the viewport
	in Direct3D, so no change relative to OpenGL conventions is needed there.
	Other state related to screen-space coordinates may need to be modified
	for the application to map from Direct3D to OpenGL window coordinate
	conventions.  For example, the viewport rectangle in Direct3D needs to be
	inverted within the window to work properly in OpenGL windowed rendering:
	
	   glViewport(d3d_viewport_x,
	              window_height - (d3d_viewport_y + d3d_viewport_height),
	              d3d_viewport_width, d3d_viewport_height);
	
	When rendering Direct3D content into a framebuffer object in OpenGL, there
	is one complication -- how to get a correct image *out* of the related
	textures.  Direct3D applications would expect a texture coordinate of
	(0,0) to correspond to the upper-left corner of a rendered image, while
	OpenGL FBO conventions would map (0,0) to the lower-left corner of the
	rendered image.  For applications wishing to use Direct3D content with
	unmodified texture coordinates, the command
	
	    glClipControl(GL_UPPER_LEFT, GL_ZERO_TO_ONE);
	
	configures the OpenGL to invert geometry vertically inside the viewport.
	Content at the top of the viewport for Direct3D will be rendered to the
	bottom of the viewport from the point of view of OpenGL, but will have a
	<t> texture coordinate of zero in both cases.  When operating in this
	mode, applications need not invert the programmed viewport rectangle as
	recommended for windowed rendering above.
	
	Applications happy with OpenGL's origin conventions but seeking
	potentially improved depth precision can configure clip controls using:
	
	    glClipControl(GL_LOWER_LEFT, GL_ZERO_TO_ONE);
	
	to avoid the loss of precision from the DepthRange transformation
	(which by default is z_window = z_ndc * 0.5 + 0.5).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/clip_control.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.clip_control import *
from OpenGL.raw.GL.ARB.clip_control import _EXTENSION_NAME

def glInitClipControlARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.index_material

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.index_material to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extends color index lighting to include a way for the current
	index to contribute to the color index produced by lighting.  This
	works much like ColorMaterial does for RGBA lighting by allowing
	one or more color index material properties to be attached to the
	current index.
	
	The color index lighting formula is also modified so that the lit
	color index may be bitwise shifted in order to allow greater control
	when using lighting and fog together in color index mode.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/index_material.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.index_material import *
from OpenGL.raw.GL.EXT.index_material import _EXTENSION_NAME

def glInitIndexMaterialEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension AMD.shader_explicit_vertex_parameter

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.shader_explicit_vertex_parameter to provide a more 
Python-friendly API

Overview (from the spec)
	
	Unextended GLSL provides a set of fixed function interpolation modes and
	even those are limited to certain types of interpolants (for example,
	interpolation of integer and double isn't supported).
	
	This extension introduces new built-in functions allowing access to vertex
	parameters explicitly in  the fragment shader. It also exposes barycentric
	coordinates as new built-in variables, which can be used to implement
	custom interpolation algorithms using shader code.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/shader_explicit_vertex_parameter.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.shader_explicit_vertex_parameter import *
from OpenGL.raw.GL.AMD.shader_explicit_vertex_parameter import _EXTENSION_NAME

def glInitShaderExplicitVertexParameterAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.sample_shading

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.sample_shading to provide a more 
Python-friendly API

Overview (from the spec)
	
	In standard multisample rendering, an implementation is allowed to
	assign the same sets of fragment shader input values to each sample.
	This can cause aliasing where the fragment shader input values are
	used to generate a result that doesn't antialias itself, for example
	with alpha-tested transparency.
	
	This extension adds the ability to explicitly request that an
	implementation use a minimum number of unique set of fragment
	computation inputs when multisampling a pixel. Specifying such a
	requirement can reduce aliasing that results from evaluating the
	fragment computations too few times per pixel.
	
	This extension adds new global state that controls the minimum
	number of samples for which attribute data is independently
	interpolated. When enabled, all fragment-shading operations
	are executed independently on each sample.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/sample_shading.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.sample_shading import *
from OpenGL.raw.GLES2.OES.sample_shading import _EXTENSION_NAME

def glInitSampleShadingOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension INTEL.blackhole_render

This module customises the behaviour of the 
OpenGL.raw.GLES2.INTEL.blackhole_render to provide a more 
Python-friendly API

Overview (from the spec)
	
	The purpose of this extension is to allow an application to disable all
	rendering operations emitted to the GPU through the OpenGL rendering
	commands (Draw*, DispatchCompute*, BlitFramebuffer, etc...). Changes to the
	OpenGL pipeline are not affected.
	
	New preprocessor #defines are added to the OpenGL Shading Language:
	
	  #define GL_INTEL_blackhole_render 1

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/INTEL/blackhole_render.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.INTEL.blackhole_render import *
from OpenGL.raw.GLES2.INTEL.blackhole_render import _EXTENSION_NAME

def glInitBlackholeRenderINTEL():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.protected_textures

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.protected_textures to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension requires another extension like EGL_EXT_protected_content to
	have created a protected context.  A protected context enables the
	driver to put the GPU into a protected mode where it is able to operate on
	protected surfaces.
	
	This extension enables allocating standard GL textures as protected
	surfaces. Previously these textures would have had to have been created as
	special EGL surfaces.  This allows use-cases such as depth, stencil, or
	mipmapped textures to be supported as destinations for rendering within
	a protected context.
	
	An explanation of undefined behavior in this extension:  Several places
	in this extension mention undefined behavior can result, which can
	include program termination.  The reason for this is because one way
	to handle protected content is by using a protected virtual to physical
	memory translation layer.  With this sort of solution a system may generate
	read or write faults when a non-protected context tries to access the
	buffer.  Depending on the system these faults might be ignored or they might
	cause process termination.  This undefined behavior should not include
	actually allowing copying any protected content to a non-protected surface.
	
	This extension does not guarantee that the implementation abides by a
	system's digital rights management requirements.  It must be verified beyond
	the existence of this extension that the implementation of this extension is
	trustworthy according to the requirements of a content protection system.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/protected_textures.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.protected_textures import *
from OpenGL.raw.GLES2.EXT.protected_textures import _EXTENSION_NAME

def glInitProtectedTexturesEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.mesh_shader

This module customises the behaviour of the 
OpenGL.raw.GL.NV.mesh_shader to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a new mechanism allowing applications to use two
	new programmable shader types -- the task and mesh shader -- to generate
	collections of geometric primitives to be processed by fixed-function
	primitive assembly and rasterization logic.  When the task and mesh
	shaders are drawn, they replace the standard programmable vertex
	processing pipeline, including vertex array attribute fetching, vertex
	shader processing, tessellation, and the geometry shader processing.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/mesh_shader.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.mesh_shader import *
from OpenGL.raw.GL.NV.mesh_shader import _EXTENSION_NAME

def glInitMeshShaderNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.texture_filter_minmax

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_filter_minmax to provide a more 
Python-friendly API

Overview (from the spec)
	
	In unextended OpenGL, minification and magnification filters such as
	LINEAR allow texture lookups to returned a filtered texel value produced
	by computing an weighted average of a collection of texels in the
	neighborhood of the texture coordinate provided.
	
	This extension provides a new texture and sampler parameter
	(TEXTURE_REDUCTION_MODE_ARB) which allows applications to produce a
	filtered texel value by computing a component-wise minimum (MIN) or
	maximum (MAX) of the texels that would normally be averaged.  The
	reduction mode is orthogonal to the minification and magnification filter
	parameters.  The filter parameters are used to identify the set of texels
	used to produce a final filtered value; the reduction mode identifies how
	these texels are combined.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_filter_minmax.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_filter_minmax import *
from OpenGL.raw.GL.ARB.texture_filter_minmax import _EXTENSION_NAME

def glInitTextureFilterMinmaxARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.window_rectangles

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.window_rectangles to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides additional orthogonally aligned "window
	rectangles" specified in window-space coordinates that restrict
	rasterization of all primitive types (geometry, images, paths)
	and framebuffer clears.
	
	When rendering to the framebuffer of an on-screen window, these
	window rectangles are ignored so these window rectangles apply to
	rendering to non-zero framebuffer objects only.
	
	From zero to an implementation-dependent limit (specified by
	GL_MAX_WINDOW_RECTANGLES_EXT) number of window rectangles can be
	operational at once.  When one or more window rectangles are active,
	rasterized fragments can either survive if the fragment is within
	any of the operational window rectangles (GL_INCLUSIVE_EXT mode) or
	be rejected if the fragment is within any of the operational window
	rectangles (GL_EXCLUSIVE_EXT mode).
	
	These window rectangles operate orthogonally to the existing scissor
	test functionality.
	
	This extension has specification language for both OpenGL and ES so
	EXT_window_rectangles can be implemented and advertised for either
	or both API contexts.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/window_rectangles.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.window_rectangles import *
from OpenGL.raw.GLES2.EXT.window_rectangles import _EXTENSION_NAME

def glInitWindowRectanglesEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glWindowRectanglesEXT.box size not checked against 'count'
glWindowRectanglesEXT=wrapper.wrapper(glWindowRectanglesEXT).setInputArraySize(
    'box', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.shader_ballot

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shader_ballot to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides the ability for a group of invocations which
	execute in lockstep to do limited forms of cross-invocation communication
	via a group broadcast of a invocation value, or broadcast of a bitarray
	representing a predicate value from each invocation in the group.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/shader_ballot.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.shader_ballot import *
from OpenGL.raw.GL.ARB.shader_ballot import _EXTENSION_NAME

def glInitShaderBallotARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""ctypes abstraction layer

We keep rewriting functions as the main entry points change,
so let's just localise the changes here...
"""
import ctypes, logging, os, sys
_log = logging.getLogger( 'OpenGL.platform.ctypesloader' )
#_log.setLevel( logging.DEBUG )
ctypes_version = [
    int(x) for x in ctypes.__version__.split('.')
]
from ctypes import util
import OpenGL

DLL_DIRECTORY = os.path.join( os.path.dirname( OpenGL.__file__ ), 'DLLS' )

def loadLibrary( dllType, name, mode = ctypes.RTLD_GLOBAL ):
    """Load a given library by name with the given mode
    
    dllType -- the standard ctypes pointer to a dll type, such as
        ctypes.cdll or ctypes.windll or the underlying ctypes.CDLL or 
        ctypes.WinDLL classes.
    name -- a short module name, e.g. 'GL' or 'GLU'
    mode -- ctypes.RTLD_GLOBAL or ctypes.RTLD_LOCAL,
        controls whether the module resolves names via other
        modules already loaded into this process.  GL modules
        generally need to be loaded with GLOBAL flags
    
    returns the ctypes C-module object
    """
    if isinstance( dllType, ctypes.LibraryLoader ):
        dllType = dllType._dlltype
    if sys.platform.startswith('linux'):
        return _loadLibraryPosix(dllType, name, mode)
    else:
        return _loadLibraryWindows(dllType, name, mode)


def _loadLibraryPosix(dllType, name, mode):
    """Load a given library for posix systems

    The problem with util.find_library is that it does not respect linker runtime variables like
    LD_LIBRARY_PATH.

    Also we cannot rely on libGLU.so to be available, for example. Most of Linux distributions will
    ship only libGLU.so.1 by default. Files ending with .so are normally used when compiling and are
    provided by dev packages.

    returns the ctypes C-module object
    """
    prefix = 'lib'
    suffix = '.so'
    base_name = prefix + name + suffix
    
    filenames_to_try = []
    # If a .so is missing, let's try libs with so version (e.g libGLU.so.9, libGLU.so.8 and so on)
    filenames_to_try.extend(list(reversed([
        base_name + '.%i' % i for i in range(0, 10)
    ])))
    err = None

    for filename in filenames_to_try:
        try:
            result = dllType(filename, mode)
            _log.debug( 'Loaded %s => %s %s', base_name, filename, result)
            return result
        except Exception as current_err:
            err = current_err
    
    _log.info('''Failed to load library ( %r ): %s''', filename, err or 'No filenames available to guess?')

def _loadLibraryWindows(dllType, name, mode):
    """Load a given library for Windows systems

    returns the ctypes C-module object
    """
    fullName = None
    try:
        fullName = util.find_library( name )
        if fullName is not None:
            name = fullName
        elif os.path.isfile( os.path.join( DLL_DIRECTORY, name + '.dll' )):
            name = os.path.join( DLL_DIRECTORY, name + '.dll' )
    except Exception as err:
        _log.info( '''Failed on util.find_library( %r ): %s''', name, err )
        # Should the call fail, we just try to load the base filename...
        pass
    try:
        return dllType( name, mode )
    except Exception as err:
        err.args += (name,fullName)
        raise

def buildFunction( functionType, name, dll ):
    """Abstract away the ctypes function-creation operation"""
    return functionType( (name, dll), )



'''OpenGL extension IMG.bindless_texture

This module customises the behaviour of the 
OpenGL.raw.GLES2.IMG.bindless_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows OpenGL ES applications to access texture objects in
	shaders without first binding each texture to one of a limited number of
	texture image units.  Using this extension, an application can query a
	64-bit unsigned integer texture handle for each texture that it wants to
	access and then use that handle directly in GLSL ES. This extensions
	significantly reduces the amount of API and internal GL driver overhead
	needed to manage resource bindings.
	
	This extension adds no new data types to GLSL.  Instead, it uses existing
	sampler data types and allows them to be populated with texture handles.
	This extension also permits sampler types to be used as uniform block
	members as well as default uniforms. Additionally, new APIs are provided to
	load values for sampler uniforms with 64-bit handle inputs.  The use of
	existing integer-based Uniform* APIs is still permitted, in which case the
	integer specified will identify a texture image.  For samplers with values
	specified as texture image units, the GL implementation will translate the
	unit number to an internal handle as required.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/IMG/bindless_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.IMG.bindless_texture import *
from OpenGL.raw.GLES2.IMG.bindless_texture import _EXTENSION_NAME

def glInitBindlessTextureIMG():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glUniformHandleui64vIMG.value size not checked against count
glUniformHandleui64vIMG=wrapper.wrapper(glUniformHandleui64vIMG).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformHandleui64vIMG.values size not checked against count
glProgramUniformHandleui64vIMG=wrapper.wrapper(glProgramUniformHandleui64vIMG).setInputArraySize(
    'values', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_NV_polygon_mode'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_NV_polygon_mode',error_checker=_errors._error_checker)
GL_FILL_NV=_C('GL_FILL_NV',0x1B02)
GL_LINE_NV=_C('GL_LINE_NV',0x1B01)
GL_POINT_NV=_C('GL_POINT_NV',0x1B00)
GL_POLYGON_MODE_NV=_C('GL_POLYGON_MODE_NV',0x0B40)
GL_POLYGON_OFFSET_LINE_NV=_C('GL_POLYGON_OFFSET_LINE_NV',0x2A02)
GL_POLYGON_OFFSET_POINT_NV=_C('GL_POLYGON_OFFSET_POINT_NV',0x2A01)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glPolygonModeNV(face,mode):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_shader_pixel_local_storage2'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_shader_pixel_local_storage2',error_checker=_errors._error_checker)
GL_FRAMEBUFFER_INCOMPLETE_INSUFFICIENT_SHADER_COMBINED_LOCAL_STORAGE_EXT=_C('GL_FRAMEBUFFER_INCOMPLETE_INSUFFICIENT_SHADER_COMBINED_LOCAL_STORAGE_EXT',0x9652)
GL_MAX_SHADER_COMBINED_LOCAL_STORAGE_FAST_SIZE_EXT=_C('GL_MAX_SHADER_COMBINED_LOCAL_STORAGE_FAST_SIZE_EXT',0x9650)
GL_MAX_SHADER_COMBINED_LOCAL_STORAGE_SIZE_EXT=_C('GL_MAX_SHADER_COMBINED_LOCAL_STORAGE_SIZE_EXT',0x9651)
@_f
@_p.types(None,_cs.GLsizei,_cs.GLsizei,arrays.GLuintArray)
def glClearPixelLocalStorageuiEXT(offset,n,values):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei)
def glFramebufferPixelLocalStorageSizeEXT(target,size):pass
@_f
@_p.types(_cs.GLsizei,_cs.GLuint)
def glGetFramebufferPixelLocalStorageSizeEXT(target):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NVX_linked_gpu_multicast'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NVX_linked_gpu_multicast',error_checker=_errors._error_checker)
GL_LGPU_SEPARATE_STORAGE_BIT_NVX=_C('GL_LGPU_SEPARATE_STORAGE_BIT_NVX',0x0800)
GL_MAX_LGPU_GPUS_NVX=_C('GL_MAX_LGPU_GPUS_NVX',0x92BA)
@_f
@_p.types(None,_cs.GLuint,_cs.GLbitfield,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei)
def glLGPUCopyImageSubDataNVX(sourceGpu,destinationGpuMask,srcName,srcTarget,srcLevel,srcX,srxY,srcZ,dstName,dstTarget,dstLevel,dstX,dstY,dstZ,width,height,depth):pass
@_f
@_p.types(None,)
def glLGPUInterlockNVX():pass
@_f
@_p.types(None,_cs.GLbitfield,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr,ctypes.c_void_p)
def glLGPUNamedBufferSubDataNVX(gpuMask,buffer,offset,size,data):pass



'''OpenGL extension SGIX.depth_pass_instrument

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.depth_pass_instrument to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines an instrument that uses the API defined in
	SGIX_instruments.  The instrument specified by this extension is a
	counter of the number of fragments which passed the Z test during
	rasterization.  The maximum value of the counter is an
	implementation-dependent constant.
	
	Some systems may maintain counters on different parts of the
	system.  For example, a system with a frame buffer distributed
	across multiple chips may maintain a count of the fragments which
	passed the depth test on each individual chip.  In this extension,
	a queriable constant is defined that indicates the number of 
	responses to expect when a measurement is taken.  This 
	mechanism allows GL implementations to be as efficient as possible.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/depth_pass_instrument.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.depth_pass_instrument import *
from OpenGL.raw.GL.SGIX.depth_pass_instrument import _EXTENSION_NAME

def glInitDepthPassInstrumentSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_ARB_create_context'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_ARB_create_context',error_checker=_errors._error_checker)
ERROR_INVALID_VERSION_ARB=_C('ERROR_INVALID_VERSION_ARB',0x2095)
WGL_CONTEXT_DEBUG_BIT_ARB=_C('WGL_CONTEXT_DEBUG_BIT_ARB',0x00000001)
WGL_CONTEXT_FLAGS_ARB=_C('WGL_CONTEXT_FLAGS_ARB',0x2094)
WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB=_C('WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB',0x00000002)
WGL_CONTEXT_LAYER_PLANE_ARB=_C('WGL_CONTEXT_LAYER_PLANE_ARB',0x2093)
WGL_CONTEXT_MAJOR_VERSION_ARB=_C('WGL_CONTEXT_MAJOR_VERSION_ARB',0x2091)
WGL_CONTEXT_MINOR_VERSION_ARB=_C('WGL_CONTEXT_MINOR_VERSION_ARB',0x2092)
@_f
@_p.types(_cs.HGLRC,_cs.HDC,_cs.HGLRC,ctypes.POINTER(_cs.c_int))
def wglCreateContextAttribsARB(hDC,hShareContext,attribList):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_index_array_formats'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_index_array_formats',error_checker=_errors._error_checker)
GL_IUI_N3F_V2F_EXT=_C('GL_IUI_N3F_V2F_EXT',0x81AF)
GL_IUI_N3F_V3F_EXT=_C('GL_IUI_N3F_V3F_EXT',0x81B0)
GL_IUI_V2F_EXT=_C('GL_IUI_V2F_EXT',0x81AD)
GL_IUI_V3F_EXT=_C('GL_IUI_V3F_EXT',0x81AE)
GL_T2F_IUI_N3F_V2F_EXT=_C('GL_T2F_IUI_N3F_V2F_EXT',0x81B3)
GL_T2F_IUI_N3F_V3F_EXT=_C('GL_T2F_IUI_N3F_V3F_EXT',0x81B4)
GL_T2F_IUI_V2F_EXT=_C('GL_T2F_IUI_V2F_EXT',0x81B1)
GL_T2F_IUI_V3F_EXT=_C('GL_T2F_IUI_V3F_EXT',0x81B2)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_memory_object_win32'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_memory_object_win32',error_checker=_errors._error_checker)
GL_DEVICE_LUID_EXT=_C('GL_DEVICE_LUID_EXT',0x9599)
GL_DEVICE_NODE_MASK_EXT=_C('GL_DEVICE_NODE_MASK_EXT',0x959A)
GL_HANDLE_TYPE_D3D11_IMAGE_EXT=_C('GL_HANDLE_TYPE_D3D11_IMAGE_EXT',0x958B)
GL_HANDLE_TYPE_D3D11_IMAGE_KMT_EXT=_C('GL_HANDLE_TYPE_D3D11_IMAGE_KMT_EXT',0x958C)
GL_HANDLE_TYPE_D3D12_RESOURCE_EXT=_C('GL_HANDLE_TYPE_D3D12_RESOURCE_EXT',0x958A)
GL_HANDLE_TYPE_D3D12_TILEPOOL_EXT=_C('GL_HANDLE_TYPE_D3D12_TILEPOOL_EXT',0x9589)
GL_HANDLE_TYPE_OPAQUE_WIN32_EXT=_C('GL_HANDLE_TYPE_OPAQUE_WIN32_EXT',0x9587)
GL_HANDLE_TYPE_OPAQUE_WIN32_KMT_EXT=_C('GL_HANDLE_TYPE_OPAQUE_WIN32_KMT_EXT',0x9588)
GL_LUID_SIZE_EXT=_C('GL_LUID_SIZE_EXT',8)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint64,_cs.GLenum,ctypes.c_void_p)
def glImportMemoryWin32HandleEXT(memory,size,handleType,handle):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint64,_cs.GLenum,ctypes.c_void_p)
def glImportMemoryWin32NameEXT(memory,size,handleType,name):pass



'''OpenGL extension VIV.shader_binary

This module customises the behaviour of the 
OpenGL.raw.GLES2.VIV.shader_binary to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables loading precompiled binary shaders compatible with
	chips designed by Vivante Corporation. 

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VIV/shader_binary.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.VIV.shader_binary import *
from OpenGL.raw.GLES2.VIV.shader_binary import _EXTENSION_NAME

def glInitShaderBinaryVIV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.vertex_shader

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.vertex_shader to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds programmable vertex level processing to OpenGL. The
	application can write vertex shaders in a high level language as defined
	in the OpenGL Shading Language specification. The language itself is not
	discussed here. A vertex shader replaces the transformation, texture
	coordinate generation and lighting parts of OpenGL, and it also adds
	texture access at the vertex level. Furthermore, management of vertex
	shader objects and loading generic attributes are discussed. A vertex
	shader object, attached to a program object, can be compiled and linked
	to produce an executable that runs on the vertex processor in OpenGL.
	This extension also defines how such an executable interacts with the
	fixed functionality vertex processing of OpenGL 1.4.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/vertex_shader.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.vertex_shader import *
from OpenGL.raw.GL.ARB.vertex_shader import _EXTENSION_NAME

def glInitVertexShaderARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glVertexAttrib1fvARB=wrapper.wrapper(glVertexAttrib1fvARB).setInputArraySize(
    'v', 1
)
glVertexAttrib1svARB=wrapper.wrapper(glVertexAttrib1svARB).setInputArraySize(
    'v', 1
)
glVertexAttrib1dvARB=wrapper.wrapper(glVertexAttrib1dvARB).setInputArraySize(
    'v', 1
)
glVertexAttrib2fvARB=wrapper.wrapper(glVertexAttrib2fvARB).setInputArraySize(
    'v', 2
)
glVertexAttrib2svARB=wrapper.wrapper(glVertexAttrib2svARB).setInputArraySize(
    'v', 2
)
glVertexAttrib2dvARB=wrapper.wrapper(glVertexAttrib2dvARB).setInputArraySize(
    'v', 2
)
glVertexAttrib3fvARB=wrapper.wrapper(glVertexAttrib3fvARB).setInputArraySize(
    'v', 3
)
glVertexAttrib3svARB=wrapper.wrapper(glVertexAttrib3svARB).setInputArraySize(
    'v', 3
)
glVertexAttrib3dvARB=wrapper.wrapper(glVertexAttrib3dvARB).setInputArraySize(
    'v', 3
)
glVertexAttrib4fvARB=wrapper.wrapper(glVertexAttrib4fvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4svARB=wrapper.wrapper(glVertexAttrib4svARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4dvARB=wrapper.wrapper(glVertexAttrib4dvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4ivARB=wrapper.wrapper(glVertexAttrib4ivARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4bvARB=wrapper.wrapper(glVertexAttrib4bvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4ubvARB=wrapper.wrapper(glVertexAttrib4ubvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4usvARB=wrapper.wrapper(glVertexAttrib4usvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4uivARB=wrapper.wrapper(glVertexAttrib4uivARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NbvARB=wrapper.wrapper(glVertexAttrib4NbvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NsvARB=wrapper.wrapper(glVertexAttrib4NsvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NivARB=wrapper.wrapper(glVertexAttrib4NivARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NubvARB=wrapper.wrapper(glVertexAttrib4NubvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NusvARB=wrapper.wrapper(glVertexAttrib4NusvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NuivARB=wrapper.wrapper(glVertexAttrib4NuivARB).setInputArraySize(
    'v', 4
)
# INPUT glVertexAttribPointerARB.pointer size not checked against 'size,type,stride'
glVertexAttribPointerARB=wrapper.wrapper(glVertexAttribPointerARB).setInputArraySize(
    'pointer', None
)
glGetActiveAttribARB=wrapper.wrapper(glGetActiveAttribARB).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'name',size=lambda x:(x,),pnameArg='maxLength',orPassIn=True
).setOutput(
    'size',size=(1,),orPassIn=True
).setOutput(
    'type',size=(1,),orPassIn=True
)
glGetVertexAttribdvARB=wrapper.wrapper(glGetVertexAttribdvARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribfvARB=wrapper.wrapper(glGetVertexAttribfvARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribivARB=wrapper.wrapper(glGetVertexAttribivARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribPointervARB=wrapper.wrapper(glGetVertexAttribPointervARB).setOutput(
    'pointer',size=(1,),orPassIn=True
)
### END AUTOGENERATED SECTION
from OpenGL._bytes import bytes, _NULL_8_BYTE, as_8_bit
from OpenGL.lazywrapper import lazy as _lazy
from OpenGL.GL.ARB.shader_objects import glGetObjectParameterivARB

@_lazy( glGetActiveAttribARB )
def glGetActiveAttribARB(baseOperation, program, index):
    """Retrieve the name, size and type of the uniform of the index in the program"""
    max_index = int(glGetObjectParameterivARB( program, GL_OBJECT_ACTIVE_ATTRIBUTES_ARB ))
    length = int(glGetObjectParameterivARB( program, GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB))
    if index < max_index and index >= 0 and length > 0:
        length,name,size,type = baseOperation( program, index )
        if hasattr(name,'tostring'):
            name = name.tostring().rstrip(b'\000')
        elif hasattr(name,'value'):
            name = name.value
        return name,size,type
    raise IndexError('index out of range from zero to %i' % (max_index - 1, ))

@_lazy( glGetAttribLocationARB )
def glGetAttribLocationARB( baseOperation, program, name ):
    """Check that name is a string with a null byte at the end of it"""
    if not name:
        raise ValueError( """Non-null name required""" )
    name = as_8_bit( name )
    if name[-1] != _NULL_8_BYTE:
        name = name + _NULL_8_BYTE
    return baseOperation( program, name )



'''OpenGL extension NV.shader_atomic_fp16_vector

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.shader_atomic_fp16_vector to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides GLSL built-in functions and assembly opcodes
	allowing shaders to perform a limited set of atomic read-modify-write
	operations to buffer or texture memory with 16-bit floating point vector
	surface formats.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/shader_atomic_fp16_vector.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.shader_atomic_fp16_vector import *
from OpenGL.raw.GLES2.NV.shader_atomic_fp16_vector import _EXTENSION_NAME

def glInitShaderAtomicFp16VectorNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.external_buffer

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.external_buffer to provide a more 
Python-friendly API

Overview (from the spec)
	
	Extension EXT_buffer_storage introduced immutable storage buffers to
	OpenGL ES. This extension allows the data store for an immutable buffer to
	be sourced from an external EGLClientBuffer, allowing sharing of EGL client
	buffers across APIs, across processes, and across different processing
	cores such as the GPU, CPU, and DSP.
	
	Operations can then be performed on the external buffer using standard
	GL buffer object procedures. The data in the allocation is not copied to
	the buffer object's data store; the external allocation represents a single
	memory allocation that can be shared across multiple GL objects -- this
	aspect is similar to EGL external images.  On the other hand, the external
	buffer does not provide lifetime guarantees including orphaning and sibling
	behavior as provided by EGL external images.
	
	The EGLClientBuffer must be allocated in a way which permits this shared
	access. For example, on Android via a shareable Android hardware buffer.
	This extension does not enable support for arbitrary EGLClientBuffers to be
	used as an external buffer.
	
	It is the application's responsibility to ensure synchronization between
	operations performed by separate components (DSP / CPU / GPU) and processes
	on the external buffer. Additionally the application is responsible for
	avoiding violating existing GL spec requirements. For example, mapping a
	single shared allocation to two GL buffer objects and then performing
	CopyBufferSubData such that the read and write regions overlap would
	violate the existing CopyBufferSubData spec regarding copies performed
	with the same buffer set for source and destination.
	
	The application must take any steps necessary to ensure memory access to
	the external buffer behaves as required by the application. For example,
	preventing compilation differences in data padding from causing data to be
	inadvertently corrupted by using defined structure alignment methods such
	as the std140 layout qualifier.  The application is responsible for
	managing the lifetime of the external buffer, ensuring that the external
	buffer is not deleted as long as there are any GL buffer objects referring
	to it.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/external_buffer.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.external_buffer import *
from OpenGL.raw.GLES2.EXT.external_buffer import _EXTENSION_NAME

def glInitExternalBufferEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.depth_texture

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.depth_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines a new depth texture format.  An important
	application of depth texture images is shadow casting, but separating
	this from the shadow extension allows for the potential use of depth
	textures in other applications such as image-based rendering or
	displacement mapping.  This extension does not define new depth-texture
	environment functions, such as filtering or applying the depth values
	computed from a texture, but leaves this to other extensions, such as
	the shadow extension.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/depth_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.depth_texture import *
from OpenGL.raw.GL.SGIX.depth_texture import _EXTENSION_NAME

def glInitDepthTextureSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.texture_compression

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_compression to provide a more 
Python-friendly API

Overview (from the spec)
	
	Compressing texture images can reduce texture memory utilization and
	improve performance when rendering textured primitives.  This extension
	allows OpenGL applications to use compressed texture images by providing:
	
	    (1) A framework upon which extensions providing specific compressed
	        image formats can be built.
	
	    (2) A set of generic compressed internal formats that allow
	        applications to specify that texture images should be stored in
	        compressed form without needing to code for specific compression
	        formats.
	
	An application can define compressed texture images by providing a texture
	image stored in a specific compressed image format.  This extension does
	not define any specific compressed image formats, but it does provide the
	mechanisms necessary to enable other extensions that do.
	
	An application can also define compressed texture images by providing an
	uncompressed texture image but specifying a compressed internal format.
	In this case, the GL will automatically compress the texture image using
	the appropriate image format.  Compressed internal formats can either be
	specific (as above) or generic.  Generic compressed internal formats are
	not actual image formats, but are instead mapped into one of the specific
	compressed formats provided by the GL (or to an uncompressed base internal
	format if no appropriate compressed format is available).  Generic
	compressed internal formats allow applications to use texture compression
	without needing to code to any particular compression algorithm.  Generic
	compressed formats allow the use of texture compression across a wide
	range of platforms with differing compression algorithms and also allow
	future GL implementations to substitute improved compression methods
	transparently.
	
	Compressed texture images can be obtained from the GL in uncompressed form
	by calling GetTexImage and in compressed form by calling
	GetCompressedTexImageARB.  Queried compressed images can be saved and
	later reused by calling CompressedTexImage[123]DARB.  Pre-compressed
	texture images do not need to be processed by the GL and should
	significantly improve texture loading performance relative to uncompressed
	images.
	
	This extension does not define specific compressed image formats (e.g.,
	S3TC, FXT1), nor does it provide means to encode or decode such images.
	To support images in a specific compressed format, a hardware vendor
	would:
	
	  (1) Provide a new extension defininig specific compressed
	      <internalformat> and <format> tokens for TexImage[123]D,
	      TexSubImage[123]D, CopyTexImage[12]D, CompressedTexImage[123]DARB,
	      CompressedTexSubImage[123]DARB, and GetCompressedTexImageARB calls.
	
	  (2) Specify the encoding of compressed images of that specific format.
	
	  (3) Specify a method for deriving the size of compressed images of that
	      specific format, using the <internalformat>, <width>, <height>,
	      <depth> parameters, and (if necessary) the compressed image itself.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_compression.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_compression import *
from OpenGL.raw.GL.ARB.texture_compression import _EXTENSION_NAME

def glInitTextureCompressionARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glCompressedTexImage3DARB.data size not checked against imageSize
glCompressedTexImage3DARB=wrapper.wrapper(glCompressedTexImage3DARB).setInputArraySize(
    'data', None
)
# INPUT glCompressedTexImage2DARB.data size not checked against imageSize
glCompressedTexImage2DARB=wrapper.wrapper(glCompressedTexImage2DARB).setInputArraySize(
    'data', None
)
# INPUT glCompressedTexImage1DARB.data size not checked against imageSize
glCompressedTexImage1DARB=wrapper.wrapper(glCompressedTexImage1DARB).setInputArraySize(
    'data', None
)
# INPUT glCompressedTexSubImage3DARB.data size not checked against imageSize
glCompressedTexSubImage3DARB=wrapper.wrapper(glCompressedTexSubImage3DARB).setInputArraySize(
    'data', None
)
# INPUT glCompressedTexSubImage2DARB.data size not checked against imageSize
glCompressedTexSubImage2DARB=wrapper.wrapper(glCompressedTexSubImage2DARB).setInputArraySize(
    'data', None
)
# INPUT glCompressedTexSubImage1DARB.data size not checked against imageSize
glCompressedTexSubImage1DARB=wrapper.wrapper(glCompressedTexSubImage1DARB).setInputArraySize(
    'data', None
)
# OUTPUT glGetCompressedTexImageARB.img COMPSIZE(target, level) 
### END AUTOGENERATED SECTION
from OpenGL.GL import images

for dimensions in (1,2,3):
    for function in ('glCompressedTexImage%sDARB','glCompressedTexSubImage%sDARB'):
        name = function%(dimensions,)
        globals()[ name ] = images.compressedImageFunction(
            globals()[ name ]
        )
        try:
            del name, function
        except NameError as err:
            pass
    try:
        del dimensions
    except NameError as err:
        pass

if glGetCompressedTexImageARB:
    def glGetCompressedTexImageARB( target, level, img=None ):
        """Retrieve a compressed texture image"""
        if img is None:
            length = glget.glGetTexLevelParameteriv(
                target, 0,
                GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB,
            )
            img = arrays.ArrayDataType.zeros( (length,), GL_1_0.GL_UNSIGNED_BYTE )
        return glGetCompressedTexImageARB(target, 0, img);



'''OpenGL extension NV.texture_rectangle_compressed

This module customises the behaviour of the 
OpenGL.raw.GL.NV.texture_rectangle_compressed to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows applications to use compressed texture formats
	with the TEXTURE_RECTANGLE texture target, removing an old limitation
	that prohibited such usage globally for rectangle textures.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/texture_rectangle_compressed.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.texture_rectangle_compressed import *
from OpenGL.raw.GL.NV.texture_rectangle_compressed import _EXTENSION_NAME

def glInitTextureRectangleCompressedNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.pixel_texture

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.pixel_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows the color components of pixel groups to be used as
	texture coordinates, effectively converting a color image into a texture
	coordinate image.  Because texture mapping is essentially a
	multidimensional table lookup, this conversion supports multidimensional
	color lookups for images.  Such multidimensional lookups can be used to
	implement very accurate color space conversions.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/pixel_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.pixel_texture import *
from OpenGL.raw.GL.SGIX.pixel_texture import _EXTENSION_NAME

def glInitPixelTextureSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.transform_feedback3

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.transform_feedback3 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension further extends the transform feedback capabilities
	provided by the EXT_transform_feedback, NV_transform_feedback, and
	NV_transform_feedback2 extensions.  Those extensions provided a new
	transform feedback mode, where selected vertex attributes can be recorded
	to a buffer object for each primitive processed by the GL.
	
	This extension provides increased flexibility in how vertex attributes can
	be written to buffer objects.  Previous extensions allowed applications to
	record a set of attributes interleaved into a single buffer object
	(interleaved mode) or to record into multiple objects, but with only a
	single attribute per buffer (separate mode).  This extension extends
	interleaved mode to write into multiple buffers, with multiple attributes
	per buffer.  This capability is supported for all three styles of
	transform feedback:
	
	  - "EXT"-style GLSL transform feedback (EXT_transform_feedback), where a
	     list of varyings is provided prior to linking a program object and is
	     used whenever that program object is used.
	
	  - "NV"-style GLSL transform feedback (NV_transform_feedback), where
	    "locations" of active varyings are queried after linking and are then
	    passed to a function that sets the active transform feedback varyings
	    for the program object.  Unlike the "EXT"-style mode, the set of
	    varyings to capture can be changed without relinking.
	
	  - Transform feedback for fixed-function or assembly vertex/geometry
	    shaders (NV_transform_feedback), where applications specify a set of
	    canonical attribute enums/numbers to capture.
	
	Additionally, this extension adds new support for multiple separate
	vertex streams.  New geometry shader functionality provided by the
	ARB_gpu_shader5 and NV_gpu_program5 extensions allows geometry shaders
	to direct each vertex arbitrarily at a specified vertex stream.  For
	example, a geometry program might write each "regular" vertex it emits
	to one vertex stream while writing some per-primitive data it computes
	to a second vertex stream.  This extension allows applications to
	choose a vertex stream for each buffer object it writes to, and allows
	the vertices written to each vertex stream to be recorded in separate
	buffer objects.  Only one stream may be selected for rasterization,
	and in the initial implementation, the geometry shader output topology
	must be POINTS if multiple streams are used.  When geometry shaders
	are not used, or when an old geometry shader not writing multiple
	streams is used, all vertices produced by the GL are directed at the
	stream numbered zero.  The set of transform feedback-related query
	targets is extended to accommodate multiple vertex streams, so it is
	possible to count the number of processed and recorded primitives for
	each stream separately.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/transform_feedback3.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.transform_feedback3 import *
from OpenGL.raw.GL.ARB.transform_feedback3 import _EXTENSION_NAME

def glInitTransformFeedback3ARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetQueryIndexediv=wrapper.wrapper(glGetQueryIndexediv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension AMD.name_gen_delete

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.name_gen_delete to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension simply creates 2 new entry-points that name generic
	creation and deletion of names.  The intent is to go away from API
	functionality that provides a create/delete function for each specific 
	object.  
	
	For example: 
	    glGenTextures/glDeleteTextures/glIsTexture
	    glGenBuffers/glDeleteBuffers/IsBuffer
	    glGenFramebuffers/glDeleteFramebuffers/IsFramebuffer
	
	Instead, everything is created using one entry-point GenNamesAMD and
	everything is now deleted with another entry-point DeleteNamesAMD with
	the appropriate identifier set.  In addition, everything can now be 
	queried with IsNameAMD.
	
	This alleviates the problem we may eventually encounter where we have
	many Gen/Delete/Is functions where 3 might suffice.  All that is needed
	in the new case is to add a valid identifier to the accepted parameters
	list.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/name_gen_delete.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.name_gen_delete import *
from OpenGL.raw.GL.AMD.name_gen_delete import _EXTENSION_NAME

def glInitNameGenDeleteAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGenNamesAMD=wrapper.wrapper(glGenNamesAMD).setOutput(
    'names',size=lambda x:(x,),pnameArg='num',orPassIn=True
)
# INPUT glDeleteNamesAMD.names size not checked against num
glDeleteNamesAMD=wrapper.wrapper(glDeleteNamesAMD).setInputArraySize(
    'names', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.reference_plane

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.reference_plane to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows a group of coplanar primitives to be rendered
	without depth-buffering artifacts.  This is accomplished by generating
	the depth values for all the primitives from a single ``reference plane''
	rather than from the primitives themselves.  This ensures that all the
	primitives in the group have exactly the same depth value at any given
	sample point, no matter what imprecision may exist in the original
	specifications of the primitives or in the GL's coordinate transformation
	process.
	
	The reference plane is defined by a four-component plane equation.
	When glReferencePlaneSGIX is called, equation is transformed by the
	transpose-adjoint of a matrix that is the complete object-coordinate
	to clip-coordinate transformation.  The resulting clip-coordinate
	coefficients are transformed by the current viewport when the reference
	plane is enabled.
	
	The reference plane is enabled and disabled with glEnable and glDisable.
	
	If the reference plane is enabled, a fragment (xf,yf,zf) will have a
	new z coordinate generated from (xf,yf) by giving it the same z value
	that the reference plane would have at (xf,yf).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/reference_plane.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.reference_plane import *
from OpenGL.raw.GL.SGIX.reference_plane import _EXTENSION_NAME

def glInitReferencePlaneSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glReferencePlaneSGIX=wrapper.wrapper(glReferencePlaneSGIX).setInputArraySize(
    'equation', 4
)
### END AUTOGENERATED SECTION